@@ -2,7 +2,7 @@
  *  arch-tag: Implementation of Rhythmbox icon loading
  *
  *  Copyright (C) 2002 Jorn Baayen
- *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
+ *  Copyright (C) 2003,2004 Colin Walters <walters@verbum.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,20 +28,15 @@
 #include "rb-file-helpers.h"
 #include "rb-stock-icons.h"
 
+static GtkIconFactory *factory = NULL;
+
 void
 rb_stock_icons_init (void)
 {
-	GtkIconFactory *factory;
 	int i;
 
 	static const char *items[] =
 	{
-		RB_STOCK_PLAY,
-		RB_STOCK_PAUSE,
-		RB_STOCK_STOP,
-		RB_STOCK_PREVIOUS,
-		RB_STOCK_NEXT,
-		RB_STOCK_REWIND,
 		RB_STOCK_SHUFFLE,
 		RB_STOCK_REPEAT,
 		RB_STOCK_VISUALS,
@@ -49,6 +44,7 @@ rb_stock_icons_init (void)
 		RB_STOCK_PLAYLIST_MENU,
 		RB_STOCK_LIBRARY,
 		RB_STOCK_IRADIO,
+		RB_STOCK_PLAYBACK_ERROR,
 		RB_STOCK_PLAYING,
 		RB_STOCK_PAUSED,
 		RB_STOCK_ALBUM,
@@ -63,9 +59,12 @@ rb_stock_icons_init (void)
 		RB_STOCK_UNSET_STAR,
 		RB_STOCK_NO_STAR,
 		RB_STOCK_DND_ICON,
-                RB_STOCK_AUDIOCD
+                RB_STOCK_AUDIOCD,
+		RB_STOCK_IPOD
 	};
 
+	g_return_if_fail (factory == NULL);
+
 	factory = gtk_icon_factory_new ();
 	gtk_icon_factory_add_default (factory);
 
@@ -84,6 +83,15 @@ rb_stock_icons_init (void)
 		
 		g_object_unref (G_OBJECT (pixbuf));
 	}
+}
+
+
+void
+rb_stock_icons_shutdown (void)
+{
+	g_return_if_fail (factory != NULL);
+
+	gtk_icon_factory_remove_default (factory);
 
 	g_object_unref (G_OBJECT (factory));
 }
