@@ -26,6 +26,7 @@
 #include <glib/gtypes.h>
 #include <glib-object.h>
 #include <gtk/gtkimage.h>
+#include <gtk/gtkuimanager.h>
 
 G_BEGIN_DECLS
 
@@ -34,8 +35,21 @@ gboolean rb_false_function (gpointer dummy);
 
 gboolean rb_gvalue_compare (GValue *a, GValue *b);
 
+int rb_compare_gtimeval (GTimeVal *a, GTimeVal *b);
+
+void rb_gtk_action_popup_menu (GtkUIManager *uimanager, const char *path);
+
 GtkWidget *rb_image_new_from_stock (const gchar *stock_id, GtkIconSize size);
-GdkPixbuf *rb_pixbuf_new_from_stock (const char *stock_id, GtkIconSize size);
+
+gchar *rb_uri_get_mount_point (const char *uri);
+gboolean rb_uri_is_mounted (const char *uri);
+
+
+void rb_threads_init (void);
+gboolean rb_is_main_thread (void);
+
+
+
 
 G_END_DECLS
 
