@@ -29,9 +29,12 @@
 #include "rb-debug.h"
 #include "gsequence.h"
 
-static void rhythmdb_property_model_class_init (RhythmDBPropertyModelClass *klass);
 static void rhythmdb_property_model_tree_model_init (GtkTreeModelIface *iface);
-static void rhythmdb_property_model_init (RhythmDBPropertyModel *shell_player);
+
+G_DEFINE_TYPE_WITH_CODE(RhythmDBPropertyModel, rhythmdb_property_model, G_TYPE_OBJECT,
+			G_IMPLEMENT_INTERFACE(GTK_TYPE_TREE_MODEL,
+					      rhythmdb_property_model_tree_model_init))
+
 static void rhythmdb_property_model_finalize (GObject *object);
 static void rhythmdb_property_model_set_property (GObject *object,
 					       guint prop_id,
@@ -49,9 +52,9 @@ static void rhythmdb_property_model_prop_changed_cb (RhythmDB *db, RhythmDBEntry
 						     RhythmDBPropType prop, const GValue *old,
 						     const GValue *new,
 						     RhythmDBPropertyModel *propmodel);
-static void rhythmdb_property_model_row_deleted_cb (GtkTreeModel *model,
-						    GtkTreePath *path,
-						    RhythmDBPropertyModel *propmodel);
+static void rhythmdb_property_model_entry_removed_cb (RhythmDBQueryModel *model,
+						      RhythmDBEntry *entry,
+						      RhythmDBPropertyModel *propmodel);
 static void rhythmdb_property_model_insert (RhythmDBPropertyModel *model, RhythmDBEntry *entry);
 static void rhythmdb_property_model_delete (RhythmDBPropertyModel *model,
 					    RhythmDBEntry *entry);
@@ -128,55 +131,13 @@ enum
 	PROP_QUERY_MODEL,
 };
 
-static GObjectClass *parent_class = NULL;
-
 static guint rhythmdb_property_model_signals[LAST_SIGNAL] = { 0 };
 
-GType
-rhythmdb_property_model_get_type (void)
-{
-	static GType rhythmdb_property_model_type = 0;
-	static const GInterfaceInfo tree_model_info =
-	{
-		(GInterfaceInitFunc) rhythmdb_property_model_tree_model_init,
-		NULL,
-		NULL
-	};
-
-	if (rhythmdb_property_model_type == 0)
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (RhythmDBPropertyModelClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rhythmdb_property_model_class_init,
-			NULL,
-			NULL,
-			sizeof (RhythmDBPropertyModel),
-			0,
-			(GInstanceInitFunc) rhythmdb_property_model_init
-		};
-
-		rhythmdb_property_model_type = g_type_register_static (G_TYPE_OBJECT,
-								       "RhythmDBPropertyModel",
-								       &our_info, 0);
-
-		g_type_add_interface_static (rhythmdb_property_model_type,
-					     GTK_TYPE_TREE_MODEL,
-					     &tree_model_info);
-	}
-
-	return rhythmdb_property_model_type;
-}
-
 static void
 rhythmdb_property_model_class_init (RhythmDBPropertyModelClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->set_property = rhythmdb_property_model_set_property;
 	object_class->get_property = rhythmdb_property_model_get_property;
 
@@ -290,7 +251,7 @@ rhythmdb_property_model_set_property (GObject *object,
 							      G_CALLBACK (rhythmdb_property_model_row_inserted_cb),
 							      model);
 			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->query_model),
-							      G_CALLBACK (rhythmdb_property_model_row_deleted_cb),
+							      G_CALLBACK (rhythmdb_property_model_entry_removed_cb),
 							      model);
 			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->query_model),
 							      G_CALLBACK (rhythmdb_property_model_prop_changed_cb),
@@ -307,8 +268,8 @@ rhythmdb_property_model_set_property (GObject *object,
 						 model,
 						 0);
 			g_signal_connect_object (G_OBJECT (model->priv->query_model),
-						 "row_deleted",
-						 G_CALLBACK (rhythmdb_property_model_row_deleted_cb),
+						 "entry-removed",
+						 G_CALLBACK (rhythmdb_property_model_entry_removed_cb),
 						 model,
 						 0);
 			g_signal_connect_object (G_OBJECT (model->priv->query_model),
@@ -403,7 +364,7 @@ rhythmdb_property_model_finalize (GObject *object)
 
 	g_free (model->priv);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rhythmdb_property_model_parent_class)->finalize (object);
 }
 
 RhythmDBPropertyModel *
@@ -420,18 +381,6 @@ entry_from_tree_iter (GtkTreeModel *model, GtkTreeIter *iter)
 	return entry;
 }
 
-static RhythmDBEntry *
-entry_from_tree_path (GtkTreeModel *model, GtkTreePath *path)
-{
-	GtkTreeIter entry_iter;
-	RhythmDBEntry *entry;
-
-	gtk_tree_model_get_iter (model, &entry_iter, path);
-	gtk_tree_model_get (model, &entry_iter, 0,
-			    &entry, -1);
-	return entry;
-}
-
 static void
 rhythmdb_property_model_row_inserted_cb (GtkTreeModel *model,
 					 GtkTreePath *path,
@@ -456,11 +405,10 @@ rhythmdb_property_model_prop_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
 }
 
 static void
-rhythmdb_property_model_row_deleted_cb (GtkTreeModel *model,
-					GtkTreePath *path,
-					RhythmDBPropertyModel *propmodel)
+rhythmdb_property_model_entry_removed_cb (RhythmDBQueryModel *model,
+					  RhythmDBEntry *entry,
+					  RhythmDBPropertyModel *propmodel)
 {
-	RhythmDBEntry *entry = entry_from_tree_path (model, path);
 	rhythmdb_property_model_delete (propmodel, entry);
 }
 
@@ -476,12 +424,8 @@ rhythmdb_property_model_insert (RhythmDBPropertyModel *model, RhythmDBEntry *ent
 	char *propstr;
 	char *sort_key;
 
-	rhythmdb_read_lock (model->priv->db);
-	propstr = g_strdup (rhythmdb_entry_get_string (model->priv->db, entry,
-						       model->priv->propid));
-	sort_key = g_strdup (rhythmdb_entry_get_string (model->priv->db, entry,
-							model->priv->sort_propid));
-	rhythmdb_read_unlock (model->priv->db);
+	propstr = g_strdup (rhythmdb_entry_get_string (entry, model->priv->propid));
+	sort_key = g_strdup (rhythmdb_entry_get_string (entry, model->priv->sort_propid));
 
 	rhythmdb_property_model_insert_prop (model, propstr, sort_key);
 	g_free (propstr);
@@ -527,9 +471,7 @@ rhythmdb_property_model_delete (RhythmDBPropertyModel *model,
 {
 	char *propstr;
 
-	rhythmdb_read_lock (model->priv->db);
-	propstr = g_strdup (rhythmdb_entry_get_string (model->priv->db, entry, model->priv->propid));
-	rhythmdb_read_unlock (model->priv->db);
+	propstr = g_strdup (rhythmdb_entry_get_string (entry, model->priv->propid));
 	
 	rhythmdb_property_model_delete_prop (model, propstr);
 	g_free (propstr);
