@@ -29,15 +29,22 @@
 
 #include "rhythmdb-query-model.h"
 #include "rb-debug.h"
-#include "rb-thread-helpers.h"
 #include "gsequence.h"
 #include "rb-tree-dnd.h"
-#include "rb-marshal.h"
+#include "rhythmdb-marshal.h"
 
-static void rhythmdb_query_model_class_init (RhythmDBQueryModelClass *klass);
 static void rhythmdb_query_model_tree_model_init (GtkTreeModelIface *iface);
 static void rhythmdb_query_model_drag_source_init (RbTreeDragSourceIface *iface);
 static void rhythmdb_query_model_drag_dest_init (RbTreeDragDestIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE(RhythmDBQueryModel, rhythmdb_query_model, G_TYPE_OBJECT,
+			G_IMPLEMENT_INTERFACE(GTK_TYPE_TREE_MODEL,
+					      rhythmdb_query_model_tree_model_init)
+			G_IMPLEMENT_INTERFACE(RB_TYPE_TREE_DRAG_SOURCE,
+					      rhythmdb_query_model_drag_source_init)
+			G_IMPLEMENT_INTERFACE(RB_TYPE_TREE_DRAG_DEST,
+					      rhythmdb_query_model_drag_dest_init))
+
 static void rhythmdb_query_model_init (RhythmDBQueryModel *shell_player);
 static GObject *rhythmdb_query_model_constructor (GType type, guint n_construct_properties,
 						  GObjectConstructParam *construct_properties);
@@ -52,16 +59,16 @@ static void rhythmdb_query_model_get_property (GObject *object,
 					       GParamSpec *pspec);
 static void rhythmdb_query_model_do_insert (RhythmDBQueryModel *model,
 					    RhythmDBEntry *entry);
-static void rhythmdb_query_model_do_delete (RhythmDBQueryModel *model,
-					    RhythmDBEntry *entry);
 static void rhythmdb_query_model_entry_added_cb (RhythmDB *db, RhythmDBEntry *entry,
 						 RhythmDBQueryModel *model);
 static void rhythmdb_query_model_entry_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
-						   RhythmDBPropType prop, const GValue *old,
-						   const GValue *new, RhythmDBQueryModel *model);
+						   GSList *changes, RhythmDBQueryModel *model);
 static void rhythmdb_query_model_entry_deleted_cb (RhythmDB *db, RhythmDBEntry *entry,
 						   RhythmDBQueryModel *model);
 
+static void rhythmdb_query_model_filter_out_entry (RhythmDBQueryModel *model,
+						   RhythmDBEntry *entry);
+static void rhythmdb_query_model_do_reorder (RhythmDBQueryModel *model, RhythmDBEntry *entry);
 static gboolean rhythmdb_query_model_drag_data_get (RbTreeDragSource *dragsource,
 							  GList *paths,
 							  GtkSelectionData *selection_data);
@@ -118,17 +125,10 @@ struct RhythmDBQueryModelUpdate
 {
 	enum {
 		RHYTHMDB_QUERY_MODEL_UPDATE_ROWS_INSERTED,
-		RHYTHMDB_QUERY_MODEL_UPDATE_ROW_CHANGED,
-		RHYTHMDB_QUERY_MODEL_UPDATE_ROW_DELETED,
 		RHYTHMDB_QUERY_MODEL_UPDATE_QUERY_COMPLETE,
 	} type;
 	RhythmDBEntry *entry;
 	GPtrArray *entries;
-
-	/* Only used for _ROW_CHANGED */
-	RhythmDBPropType prop;
-	GValue old;
-	GValue new;
 };
 
 struct RhythmDBQueryModelPrivate
@@ -137,12 +137,13 @@ struct RhythmDBQueryModelPrivate
 
 	GCompareDataFunc sort_func;
 	gpointer sort_user_data;
+	GDestroyNotify sort_destroy_notify;
 
 	GPtrArray *query;
 
 	guint stamp;
 
-	guint max_size_mb;
+	GnomeVFSFileSize max_size;
 	guint max_count;
 
 	gboolean cancelled;
@@ -156,6 +157,8 @@ struct RhythmDBQueryModelPrivate
 
 	GSequence *entries;
 	GHashTable *reverse_map;
+	GSequence *limited_entries;
+	GHashTable *limited_reverse_map;
 
 	GAsyncQueue *query_complete;
 
@@ -172,6 +175,7 @@ enum
 	PROP_QUERY,
 	PROP_SORT_FUNC,
 	PROP_SORT_DATA,
+	PROP_SORT_DATA_DESTROY,
 	PROP_MAX_SIZE,
 	PROP_MAX_COUNT,
 };
@@ -180,79 +184,17 @@ enum
 {
 	COMPLETE,
 	ENTRY_PROP_CHANGED,
+	ENTRY_REMOVED,
 	LAST_SIGNAL
 };
 
 static guint rhythmdb_query_model_signals[LAST_SIGNAL] = { 0 };
 
-static GObjectClass *parent_class = NULL;
-
-GType
-rhythmdb_query_model_get_type (void)
-{
-	static GType rhythmdb_query_model_type = 0;
-
-	if (rhythmdb_query_model_type == 0)
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (RhythmDBQueryModelClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rhythmdb_query_model_class_init,
-			NULL,
-			NULL,
-			sizeof (RhythmDBQueryModel),
-			0,
-			(GInstanceInitFunc) rhythmdb_query_model_init
-		};
-
-		static const GInterfaceInfo tree_model_info =
-		{
-			(GInterfaceInitFunc) rhythmdb_query_model_tree_model_init,
-			NULL,
-			NULL
-		};
-
-		static const GInterfaceInfo drag_source_info = {
-			(GInterfaceInitFunc) rhythmdb_query_model_drag_source_init,
-			NULL,
-			NULL
-		};
-
-		static const GInterfaceInfo drag_dest_info = {
-			(GInterfaceInitFunc) rhythmdb_query_model_drag_dest_init,
-			NULL,
-			NULL
-		};
-
-		rhythmdb_query_model_type = g_type_register_static (G_TYPE_OBJECT,
-								    "RhythmDBQueryModel",
-								    &our_info, 0);
-
-		g_type_add_interface_static (rhythmdb_query_model_type,
-					     GTK_TYPE_TREE_MODEL,
-					     &tree_model_info);
-
-		g_type_add_interface_static (rhythmdb_query_model_type,
-					     RB_TYPE_TREE_DRAG_SOURCE,
-					     &drag_source_info);
-
-		g_type_add_interface_static (rhythmdb_query_model_type,
-					     RB_TYPE_TREE_DRAG_DEST,
-					     &drag_dest_info);
-	}
-
-	return rhythmdb_query_model_type;
-}
-
 static void
 rhythmdb_query_model_class_init (RhythmDBQueryModelClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	if (!rhythmdb_query_model_drag_target_list)
 		rhythmdb_query_model_drag_target_list
 			= gtk_target_list_new (rhythmdb_query_model_drag_types,
@@ -293,6 +235,13 @@ rhythmdb_query_model_class_init (RhythmDBQueryModelClass *klass)
 							      "Sort user data",
 							       G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+					 PROP_SORT_DATA_DESTROY,
+					 g_param_spec_pointer ("sort-data-destroy",
+							      "GDestroyNotify",
+							      "Sort data destroy function",
+							       G_PARAM_READWRITE));
+
 	g_object_class_install_property (object_class,
 					 PROP_MAX_SIZE,
 					 g_param_spec_int ("max-size",
@@ -315,9 +264,17 @@ rhythmdb_query_model_class_init (RhythmDBQueryModelClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RhythmDBQueryModelClass, entry_prop_changed),
 			      NULL, NULL,
-			      rb_marshal_VOID__POINTER_INT_POINTER_POINTER,
+			      rhythmdb_marshal_VOID__POINTER_INT_POINTER_POINTER,
 			      G_TYPE_NONE, 4, G_TYPE_POINTER,
 			      G_TYPE_INT, G_TYPE_POINTER, G_TYPE_POINTER);
+	rhythmdb_query_model_signals[ENTRY_REMOVED] =
+		g_signal_new ("entry-removed",
+			      RHYTHMDB_TYPE_QUERY_MODEL,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RhythmDBQueryModelClass, entry_removed),
+			      NULL, NULL,
+			      rhythmdb_marshal_VOID__POINTER,
+			      G_TYPE_NONE, 1, G_TYPE_POINTER);
 	rhythmdb_query_model_signals[COMPLETE] =
 		g_signal_new ("complete",
 			      RHYTHMDB_TYPE_QUERY_MODEL,
@@ -385,8 +342,11 @@ rhythmdb_query_model_set_property (GObject *object,
 	case PROP_SORT_DATA:
 		model->priv->sort_user_data = g_value_get_pointer (value);
 		break;
+	case PROP_SORT_DATA_DESTROY:
+		model->priv->sort_destroy_notify  = g_value_get_pointer (value);
+		break;
 	case PROP_MAX_SIZE:
-		model->priv->max_size_mb = g_value_get_int (value);
+		model->priv->max_size = g_value_get_int (value) * 1024 * 1024;
 		break;
 	case PROP_MAX_COUNT:
 		model->priv->max_count = g_value_get_int (value);
@@ -419,8 +379,11 @@ rhythmdb_query_model_get_property (GObject *object,
 	case PROP_SORT_DATA:
 		g_value_set_pointer (value, model->priv->sort_user_data);
 		break;
+	case PROP_SORT_DATA_DESTROY:
+		g_value_set_pointer (value, model->priv->sort_destroy_notify);
+		break;
 	case PROP_MAX_SIZE:
-		g_value_set_int (value, model->priv->max_size_mb);
+		g_value_set_int (value, model->priv->max_size / (1024 * 1024));
 		break;
 	case PROP_MAX_COUNT:
 		g_value_set_int (value, model->priv->max_count);
@@ -440,6 +403,8 @@ rhythmdb_query_model_init (RhythmDBQueryModel *model)
 
 	model->priv->entries = g_sequence_new (NULL);
 	model->priv->reverse_map = g_hash_table_new (g_direct_hash, g_direct_equal);
+	model->priv->limited_entries = g_sequence_new (NULL);
+	model->priv->limited_reverse_map = g_hash_table_new (g_direct_hash, g_direct_equal);
 
 	model->priv->query_complete = g_async_queue_new ();
 	model->priv->pending_updates = g_async_queue_new ();
@@ -494,17 +459,9 @@ rhythmdb_query_model_free_pending_update (RhythmDBQueryModel *model,
 		for (i = 0; i < update->entries->len; i++)
 			rhythmdb_entry_unref (model->priv->db,
 					      g_ptr_array_index (update->entries, i));
-		break;
 		g_ptr_array_free (update->entries, TRUE);
+		break;
 	}
-	case RHYTHMDB_QUERY_MODEL_UPDATE_ROW_CHANGED:
-		rhythmdb_entry_unref (model->priv->db, update->entry);
-		g_value_unset (&update->old);
-		g_value_unset (&update->new);
-		break;
-	case RHYTHMDB_QUERY_MODEL_UPDATE_ROW_DELETED:
-		rhythmdb_entry_unref (model->priv->db, update->entry);
-		break;
 	case RHYTHMDB_QUERY_MODEL_UPDATE_QUERY_COMPLETE:
 		break;
 	}
@@ -527,10 +484,16 @@ rhythmdb_query_model_finalize (GObject *object)
 
 	rb_debug ("finalizing query model");
 
+	if (model->priv->sort_user_data &&
+	    model->priv->sort_destroy_notify)
+		model->priv->sort_destroy_notify (model->priv->sort_user_data);
+
 	g_source_remove (model->priv->model_poll_id);
 
 	g_hash_table_destroy (model->priv->reverse_map);
 	g_sequence_free (model->priv->entries);
+	g_hash_table_destroy (model->priv->limited_reverse_map);
+	g_sequence_free (model->priv->limited_entries);
 
 	if (model->priv->query)
 		rhythmdb_query_free (model->priv->query);
@@ -543,7 +506,7 @@ rhythmdb_query_model_finalize (GObject *object)
 
 	g_free (model->priv);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rhythmdb_query_model_parent_class)->finalize (object);
 }
 
 RhythmDBQueryModel *
@@ -592,13 +555,6 @@ rhythmdb_query_model_set_connected (RhythmDBQueryModel *model, gboolean connecte
 	model->priv->connected = connected;
 }
 
-void
-rhythmdb_query_model_finish_complete (RhythmDBQueryModel *model)
-{
-	if (!rb_thread_helpers_in_main_thread ())
-		g_async_queue_pop (model->priv->query_complete);
-}
-
 gboolean
 rhythmdb_query_model_has_pending_changes (RhythmDBQueryModel *model)
 {
@@ -611,68 +567,69 @@ choose_sequence_element (GSequence *seq)
 	return g_sequence_get_ptr_at_pos (seq, 0);
 }
 
-/* Threading: should be entered via database thread, with db lock held
- */
 static void
 rhythmdb_query_model_entry_added_cb (RhythmDB *db, RhythmDBEntry *entry,
 				     RhythmDBQueryModel *model)
 {
-	if (G_LIKELY (model->priv->query)) {
-		if (model->priv->max_count > 0
-		    && g_hash_table_size (model->priv->reverse_map) >= model->priv->max_count)
-			return;
-		/* Check size later */
-		
+	if (model->priv->query) {
 		if (rhythmdb_evaluate_query (db, model->priv->query, entry)) {
-			rhythmdb_query_model_add_entry (model, entry);
+			rhythmdb_query_model_do_insert (model, entry);
 		}
 	}
 }
 
 static void
 rhythmdb_query_model_entry_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
-				       RhythmDBPropType prop, const GValue *old,
-				       const GValue *new, RhythmDBQueryModel *model)
+				       GSList *changes, RhythmDBQueryModel *model)
 {
-	if (!model->priv->connected) {
+	gboolean hidden = FALSE;
+	GSList *t;
+
+	if (!model->priv->connected)
+		return;
+
+
+	hidden = rhythmdb_entry_get_boolean (entry, RHYTHMDB_PROP_HIDDEN);
+
+	if (g_hash_table_lookup (model->priv->reverse_map, entry) == NULL) {
+		if (hidden == FALSE) {
+			/* the changed entry may now satisfy the query 
+			 * so we test it */
+			rhythmdb_query_model_entry_added_cb (db, entry, model);
+		}
 		return;
 	}
 
-	if (g_hash_table_lookup (model->priv->reverse_map, entry) != NULL) {
-		struct RhythmDBQueryModelUpdate *update;
-
-		if (model->priv->query &&
-		    !rhythmdb_evaluate_query (db, model->priv->query, entry)) {
-			rhythmdb_query_model_remove_entry (model, entry);
-			return;
-		}
-
-		rb_debug ("queueing entry change");
-
-		update = g_new0 (struct RhythmDBQueryModelUpdate, 1);
-		update->type = RHYTHMDB_QUERY_MODEL_UPDATE_ROW_CHANGED;
-		update->entry = entry;
-		update->prop = prop;
-		g_value_init (&update->old, G_VALUE_TYPE (old));
-		g_value_copy (old, &update->old);
-		g_value_init (&update->new, G_VALUE_TYPE (new));
-		g_value_copy (new, &update->new);
-	
-		/* Called with a locked database */
-		rhythmdb_entry_ref_unlocked (model->priv->db, entry);
-
-		g_async_queue_push (model->priv->pending_updates, update);
-	} else {
-		/* the changed entry may now satisfy the query so we test it */
-		rhythmdb_query_model_entry_added_cb (db, entry, model);
+	if (hidden) {
+		rhythmdb_query_model_remove_entry (model, entry);
+		return;
 	}
+
+	/* emit separate change signals for each property */
+	for (t = changes; t; t = t->next) {
+		RhythmDBEntryChange *change = t->data;
+		g_signal_emit (G_OBJECT (model),
+			       rhythmdb_query_model_signals[ENTRY_PROP_CHANGED], 0,
+			       entry, change->prop, &change->old, &change->new);
+	}
+
+	if (model->priv->query &&
+	    !rhythmdb_evaluate_query (db, model->priv->query, entry)) {
+		rhythmdb_query_model_filter_out_entry (model, entry);
+		return;
+	}
+
+	/* it may have moved, so we can't just emit a changed entry */
+	rhythmdb_query_model_do_reorder (model, entry);
 }
 
 static void
 rhythmdb_query_model_entry_deleted_cb (RhythmDB *db, RhythmDBEntry *entry,
 				       RhythmDBQueryModel *model)
 {
-	rhythmdb_query_model_remove_entry (model, entry);
+	
+	if (g_hash_table_lookup (model->priv->reverse_map, entry))
+		rhythmdb_query_model_remove_entry (model, entry);
 }
 
 /* Threading: Called from the database context, holding a db write lock
@@ -683,10 +640,7 @@ rhythmdb_query_model_add_entries (RhythmDBQueryModel *model, GPtrArray *entries)
 	struct RhythmDBQueryModelUpdate *update;
 	guint i;
 
-	if (model->priv->max_count > 0
-	    && g_hash_table_size (model->priv->reverse_map) >= model->priv->max_count)
-		return;
-	/* Check size later */
+	rb_debug ("adding %d entries", entries->len);
 
 	update = g_new (struct RhythmDBQueryModelUpdate, 1);
 	update->type = RHYTHMDB_QUERY_MODEL_UPDATE_ROWS_INSERTED;
@@ -694,8 +648,7 @@ rhythmdb_query_model_add_entries (RhythmDBQueryModel *model, GPtrArray *entries)
 
 	/* Called with a locked database */
 	for (i = 0; i < update->entries->len; i++)
-		rhythmdb_entry_ref_unlocked (model->priv->db,
-					     g_ptr_array_index (update->entries, i));
+		rhythmdb_entry_ref (model->priv->db, g_ptr_array_index (update->entries, i));
 
 	g_async_queue_push (model->priv->pending_updates, update);
 }
@@ -709,24 +662,6 @@ rhythmdb_query_model_add_entry (RhythmDBQueryModel *model, RhythmDBEntry *entry)
 	rhythmdb_query_model_add_entries (model, entries);
 }
 
-
-/* Threading: Called from the database context, holding a db write lock
- */
-void
-rhythmdb_query_model_remove_entry (RhythmDBQueryModel *model, RhythmDBEntry *entry)
-{
-	struct RhythmDBQueryModelUpdate *update;
-	
-	update = g_new (struct RhythmDBQueryModelUpdate, 1);
-	update->type = RHYTHMDB_QUERY_MODEL_UPDATE_ROW_DELETED;
-	update->entry = entry;
-	
-	/* Called with a locked database */
-	rhythmdb_entry_ref_unlocked (model->priv->db, entry);
-	
-	g_async_queue_push (model->priv->pending_updates, update);
-}
-
 GnomeVFSFileSize
 rhythmdb_query_model_get_size (RhythmDBQueryModel *model)
 {
@@ -755,42 +690,16 @@ compare_times (GTimeVal *a, GTimeVal *b)
 }
 
 static void
-rhythmdb_query_model_do_insert (RhythmDBQueryModel *model,
-				RhythmDBEntry *entry)
+rhythmdb_query_model_insert_into_main_list (RhythmDBQueryModel *model, RhythmDBEntry *entry)
 {
 	GSequencePtr ptr;
-	guint64 size;
-	long duration;
-	GtkTreePath *path;
-	GtkTreeIter iter;
-
-	/* we check again if the entry already exists in the hash table */
-	if (g_hash_table_lookup (model->priv->reverse_map, entry) != NULL)
-		return;
-
-	if (model->priv->max_count > 0
-	    && g_hash_table_size (model->priv->reverse_map) >= model->priv->max_count)
-		return;
-
-	rhythmdb_read_lock (model->priv->db);
-	size = rhythmdb_entry_get_uint64 (model->priv->db, entry,
-					  RHYTHMDB_PROP_FILE_SIZE);
-	duration = rhythmdb_entry_get_long (model->priv->db,
-					    entry, RHYTHMDB_PROP_DURATION);
-	rhythmdb_read_unlock (model->priv->db);
-
-	if (model->priv->max_size_mb > 0
-	    && ((model->priv->total_size + size) / (1024*1024)) >= model->priv->max_size_mb)
-		return;
 
 	rhythmdb_entry_ref (model->priv->db, entry);
 
 	if (model->priv->sort_func) {
-		rhythmdb_read_lock (model->priv->db);
 		ptr = g_sequence_insert_sorted (model->priv->entries, entry,
 						model->priv->sort_func,
 						model->priv->sort_user_data);
-		rhythmdb_read_unlock (model->priv->db);
 	} else {
 		ptr = g_sequence_get_end_ptr (model->priv->entries);
 		g_sequence_insert (ptr, entry);
@@ -799,9 +708,217 @@ rhythmdb_query_model_do_insert (RhythmDBQueryModel *model,
 
 	g_hash_table_insert (model->priv->reverse_map, entry, ptr);
 
-	model->priv->total_duration += duration;
-	model->priv->total_size += size;
+	model->priv->total_duration += rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DURATION);
+	model->priv->total_size += rhythmdb_entry_get_uint64 (entry, RHYTHMDB_PROP_FILE_SIZE);
+}
+
+static void
+rhythmdb_query_model_insert_into_limited_list (RhythmDBQueryModel *model, RhythmDBEntry *entry)
+{
+	GSequencePtr ptr;
+
+	rhythmdb_entry_ref (model->priv->db, entry);
+
+	if (model->priv->sort_func) {
+		ptr = g_sequence_insert_sorted (model->priv->limited_entries, entry,
+						model->priv->sort_func,
+						model->priv->sort_user_data);
+	} else {
+		ptr = g_sequence_get_end_ptr (model->priv->limited_entries);
+		g_sequence_insert (ptr, entry);
+		ptr = g_sequence_ptr_prev (ptr);
+	}
+
+	g_hash_table_insert (model->priv->limited_reverse_map, entry, ptr);
+}
+
+static void
+rhythmdb_query_model_remove_from_main_list (RhythmDBQueryModel *model, RhythmDBEntry *entry)
+{
+	GSequencePtr ptr = g_hash_table_lookup (model->priv->reverse_map, entry);
+	int index;
+	GtkTreePath *path;
+
+	index = g_sequence_ptr_get_position (ptr);
 	
+	path = gtk_tree_path_new ();
+	gtk_tree_path_append_index (path, index);
+
+	gtk_tree_model_row_deleted (GTK_TREE_MODEL (model), path);
+	gtk_tree_path_free (path);
+	
+	model->priv->total_duration -= entry->duration;
+	model->priv->total_size -= entry->file_size;
+
+	g_sequence_remove (ptr);
+	g_hash_table_remove (model->priv->reverse_map, entry);
+	
+	rhythmdb_entry_unref (model->priv->db, entry);
+}
+
+static void
+rhythmdb_query_model_remove_from_limited_list (RhythmDBQueryModel *model, RhythmDBEntry *entry)
+{
+	GSequencePtr ptr = g_hash_table_lookup (model->priv->limited_reverse_map, entry);
+
+	g_sequence_remove (ptr);
+	g_hash_table_remove (model->priv->limited_reverse_map, entry);
+	rhythmdb_entry_unref (model->priv->db, entry);
+}
+
+static void
+rhythmdb_query_model_update_limited_entries (RhythmDBQueryModel *model)
+{
+	RhythmDBEntry *entry;
+	GSequencePtr ptr;
+
+	/* make it fit inside the limits */
+	while ((model->priv->max_count > 0 && g_hash_table_size (model->priv->reverse_map) > model->priv->max_count) ||
+	      (model->priv->max_size > 0 && model->priv->total_size > model->priv->max_size)) {
+
+		ptr = g_sequence_ptr_prev (g_sequence_get_end_ptr (model->priv->entries));
+		entry = (RhythmDBEntry*) g_sequence_ptr_get_data (ptr);
+
+		rb_debug ("query: moving entry to limited list");
+		rhythmdb_query_model_remove_from_main_list (model, entry);
+		rhythmdb_query_model_insert_into_limited_list (model, entry);
+	}
+
+	/* move entries that were previously limited, back to the main list */
+	while (TRUE) {
+		int size;
+		int duration;
+		GtkTreePath *path;
+		GtkTreeIter iter;
+
+		ptr = g_sequence_get_begin_ptr (model->priv->limited_entries);
+		if (!ptr || ptr == g_sequence_get_end_ptr (model->priv->limited_entries))
+			break;
+		entry = (RhythmDBEntry*) g_sequence_ptr_get_data (ptr);
+		if (!entry)
+			break;
+
+		size = entry->file_size;
+		duration = entry->duration;
+
+		if ((model->priv->max_count > 0 && (g_hash_table_size (model->priv->reverse_map) + 1) > model->priv->max_count) ||
+		    (model->priv->max_size > 0 && model->priv->total_size + size > model->priv->max_size))
+			break;
+
+		rb_debug ("query: moving entry from limited list");
+		rhythmdb_query_model_remove_from_limited_list (model, entry);
+		rhythmdb_query_model_insert_into_main_list (model, entry);
+
+		ptr = g_hash_table_lookup (model->priv->reverse_map, entry);
+		iter.stamp = model->priv->stamp;
+		iter.user_data = ptr;
+		path = rhythmdb_query_model_get_path (GTK_TREE_MODEL (model),
+						      &iter);
+		gtk_tree_model_row_inserted (GTK_TREE_MODEL (model),
+					     path, &iter);
+		gtk_tree_path_free (path);
+	}
+}
+
+
+static void
+rhythmdb_query_model_do_reorder (RhythmDBQueryModel *model, RhythmDBEntry *entry)
+{
+	GSequencePtr ptr;
+	int old_pos, new_pos;
+	gint *reorder_map;
+	int length, i;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+
+	if (model->priv->sort_func == NULL)
+		return;
+
+	ptr = g_sequence_get_begin_ptr (model->priv->limited_entries);
+
+	if (ptr != NULL && !g_sequence_ptr_is_end (ptr)) {
+		RhythmDBEntry *first_limited = g_sequence_ptr_get_data (ptr);
+		int cmp = (model->priv->sort_func) (entry, first_limited, model->priv->sort_user_data);
+
+		if (cmp > 0) {
+			/* the entry belongs in the limited list, so we don't need a re-order */
+			rhythmdb_query_model_remove_entry (model, entry);
+			rhythmdb_query_model_do_insert (model, entry);
+			return;
+		}
+	}
+
+	/* it may have moved, check for a re-order */
+	length = g_sequence_get_length (model->priv->entries);
+
+	ptr = g_hash_table_lookup (model->priv->reverse_map, entry);
+	g_hash_table_remove (model->priv->reverse_map, entry);
+	old_pos = g_sequence_ptr_get_position (ptr);
+	g_sequence_remove (ptr);
+
+	ptr = g_sequence_insert_sorted (model->priv->entries, entry,
+					model->priv->sort_func,
+					model->priv->sort_user_data);
+	new_pos = g_sequence_ptr_get_position (ptr);
+	g_hash_table_insert (model->priv->reverse_map, entry, ptr);
+
+	if (new_pos == old_pos) {
+		/* it hasn't moved, don't emit a re-order */
+		return;
+	}
+
+	reorder_map = malloc (length * sizeof(gint));
+
+	if (new_pos > old_pos ){
+		/* it has mover furthur down the list */
+		for (i = 0; i < old_pos; i++)
+			reorder_map[i] = i;
+		for (i = old_pos; i < new_pos; i++)
+			reorder_map[i] = i + 1;
+		reorder_map[new_pos] = old_pos;
+		for (i = new_pos + 1; i < length; i++)
+			reorder_map[i] = i;
+	} else {
+		/* it has moved up the list */
+		for (i = 0; i < new_pos; i++)
+			reorder_map[i] = i;
+		reorder_map[new_pos] = old_pos;
+		for (i = new_pos + 1; i < old_pos; i++)
+			reorder_map[i] = i - 1;
+		for (i = new_pos; i < length; i++)
+			reorder_map[i] = i;
+	}
+
+	gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter);
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
+
+	gtk_tree_model_rows_reordered (GTK_TREE_MODEL (model),
+				       path, &iter,
+				       reorder_map);
+	gtk_tree_path_free (path);
+	free (reorder_map);
+}
+
+static void
+rhythmdb_query_model_do_insert (RhythmDBQueryModel *model,
+				RhythmDBEntry *entry)
+{
+	GSequencePtr ptr;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+
+	/* we check again if the entry already exists in the hash table */
+	if (g_hash_table_lookup (model->priv->reverse_map, entry) != NULL)
+		return;
+
+	ptr = g_hash_table_lookup (model->priv->limited_reverse_map, entry);
+	if (ptr != NULL)
+		rhythmdb_query_model_remove_from_limited_list (model, entry);
+
+	rhythmdb_query_model_insert_into_main_list (model, entry);
+
+	/* it was added to the main list, send out the inserted signal */
+	ptr = g_hash_table_lookup (model->priv->reverse_map, entry);
 	iter.stamp = model->priv->stamp;
 	iter.user_data = ptr;
 	path = rhythmdb_query_model_get_path (GTK_TREE_MODEL (model),
@@ -809,40 +926,44 @@ rhythmdb_query_model_do_insert (RhythmDBQueryModel *model,
 	gtk_tree_model_row_inserted (GTK_TREE_MODEL (model),
 				     path, &iter);
 	gtk_tree_path_free (path);
+
+	rhythmdb_query_model_update_limited_entries (model);
 }
 
 static void
-rhythmdb_query_model_do_delete (RhythmDBQueryModel *model,
-				RhythmDBEntry *entry)
+rhythmdb_query_model_filter_out_entry (RhythmDBQueryModel *model,
+				       RhythmDBEntry *entry)
 {
-	GtkTreePath *path;
 	GSequencePtr ptr;
-	int index;
 
 	ptr = g_hash_table_lookup (model->priv->reverse_map, entry);
-
-	if (ptr == NULL)
+	if (ptr != NULL) {
+		rhythmdb_query_model_remove_from_main_list (model, entry);
+		rhythmdb_query_model_update_limited_entries (model);
 		return;
+	}
 
-	index = g_sequence_ptr_get_position (ptr);
-	
-	path = gtk_tree_path_new ();
-	gtk_tree_path_append_index (path, index);
-	rb_debug ("emitting row deleted");
-	gtk_tree_model_row_deleted (GTK_TREE_MODEL (model), path);
-	gtk_tree_path_free (path);
-	
-	rhythmdb_read_lock (model->priv->db);
-	model->priv->total_duration -= rhythmdb_entry_get_long (model->priv->db, entry,
-								RHYTHMDB_PROP_DURATION);
-	model->priv->total_size -= rhythmdb_entry_get_uint64 (model->priv->db, entry,
-							      RHYTHMDB_PROP_FILE_SIZE);
-	rhythmdb_read_unlock (model->priv->db);
+	ptr = g_hash_table_lookup (model->priv->limited_reverse_map, entry);
+	if (ptr != NULL) {
+		rhythmdb_query_model_remove_from_limited_list (model, entry);
+		rhythmdb_query_model_update_limited_entries (model);
+		return;
+	}
+}
+				
 
-	g_sequence_remove (ptr);
-	g_hash_table_remove (model->priv->reverse_map, entry);
-	
-	rhythmdb_entry_unref (model->priv->db, entry);
+void
+rhythmdb_query_model_remove_entry (RhythmDBQueryModel *model, 
+				   RhythmDBEntry *entry)
+{
+	/* emit entry-removed, so listeners know the
+	 * entry has actually been removed, rather than filtered
+	 * out.
+	 */
+	g_signal_emit (G_OBJECT (model),
+		       rhythmdb_query_model_signals[ENTRY_REMOVED], 0,
+		       entry);
+	rhythmdb_query_model_filter_out_entry (model, entry);
 }
 
 static gboolean
@@ -884,9 +1005,7 @@ rhythmdb_query_model_poll (RhythmDBQueryModel *model, GTimeVal *timeout)
 		return FALSE;
 
 	while ((update = g_async_queue_try_pop (model->priv->pending_updates)) != NULL) {
-		GtkTreePath *path;
 		GtkTreeIter iter;
-		GSequencePtr ptr;
 
 		if (update == NULL)
 			break;
@@ -899,33 +1018,8 @@ rhythmdb_query_model_poll (RhythmDBQueryModel *model, GTimeVal *timeout)
 			guint i;
 			rb_debug ("inserting %d rows", update->entries->len);
 			for (i = 0; i < update->entries->len; i++)
-				rhythmdb_query_model_do_insert (model, g_ptr_array_index (update->entries, i));
-			break;
-		}
-		case RHYTHMDB_QUERY_MODEL_UPDATE_ROW_CHANGED:
-		{
-			ptr = g_hash_table_lookup (model->priv->reverse_map,
-						   update->entry);
-
-			if (ptr == NULL)
-				break;
-
-			g_signal_emit (G_OBJECT (model),
-				       rhythmdb_query_model_signals[ENTRY_PROP_CHANGED], 0,
-				       update->entry, update->prop, &update->old, &update->new);
-
-			iter.user_data = ptr;
-			path = rhythmdb_query_model_get_path (GTK_TREE_MODEL (model),
-							      &iter);
-			rb_debug ("emitting row changed");
-			gtk_tree_model_row_changed (GTK_TREE_MODEL (model),
-						    path, &iter);
-			gtk_tree_path_free (path);
-			break;
-		}
-		case RHYTHMDB_QUERY_MODEL_UPDATE_ROW_DELETED:
-		{
-			rhythmdb_query_model_do_delete (model, update->entry);
+				rhythmdb_query_model_do_insert (model,
+								g_ptr_array_index (update->entries, i));
 			break;
 		}
 		case RHYTHMDB_QUERY_MODEL_UPDATE_QUERY_COMPLETE:
@@ -1042,21 +1136,17 @@ rhythmdb_query_model_drag_data_get (RbTreeDragSource *dragsource,
 
 			entry = g_sequence_ptr_get_data (iter.user_data);
 
-			rhythmdb_read_lock (model->priv->db);
-
-			location = rhythmdb_entry_get_string (model->priv->db, entry, RHYTHMDB_PROP_LOCATION);
+			location = entry->location;
 			g_string_append (data, location);
 
-			rhythmdb_read_unlock (model->priv->db);
-
 			if (tem->next)
 				g_string_append (data, "\r\n");
 		}
 
 		gtk_selection_data_set (selection_data,
 					selection_data->target,
-					8, data->str,
-					strlen (data->str));
+					8, (guchar *) data->str,
+					data->len);
 
 		g_string_free (data, TRUE);
 
@@ -1087,7 +1177,7 @@ rhythmdb_query_model_drag_data_received (RbTreeDragDest *drag_dest,
 		RhythmDBEntry *entry;
 		int i = 0;
 
-		strv = g_strsplit (selection_data->data, "\r\n", -1);
+		strv = g_strsplit ((char *) selection_data->data, "\r\n", -1);
 
 		if (dest == NULL || !rhythmdb_query_model_get_iter (GTK_TREE_MODEL (model), &iter, dest))
 			ptr = g_sequence_get_end_ptr (model->priv->entries);
@@ -1102,14 +1192,12 @@ rhythmdb_query_model_drag_data_received (RbTreeDragDest *drag_dest,
 			GtkTreeIter tem_iter;
 			GtkTreePath *tem_path;
 
-			rhythmdb_read_lock (model->priv->db);
 
 			entry = rhythmdb_entry_lookup_by_location (model->priv->db,
 								   strv[i]);
-			rhythmdb_read_unlock (model->priv->db);
 
 			if (entry == NULL)
-				rhythmdb_add_uri_async (model->priv->db, strv[i]);
+				rhythmdb_add_uri (model->priv->db, strv[i]);
 			else {
 				GSequencePtr old_ptr = g_hash_table_lookup (model->priv->reverse_map,
 									    entry);
@@ -1194,9 +1282,7 @@ rhythmdb_query_model_get_flags (GtkTreeModel *model)
 static gint
 rhythmdb_query_model_get_n_columns (GtkTreeModel *tree_model)
 {
-/* 	RhythmDBQueryModel *model = RHYTHMDB_QUERY_MODEL (tree_model); */
-
-	return RHYTHMDB_NUM_PROPERTIES;
+	return 2;
 }
 
 static GType
@@ -1220,18 +1306,18 @@ rhythmdb_query_model_get_iter (GtkTreeModel *tree_model, GtkTreeIter *iter,
 {
 	RhythmDBQueryModel *model = RHYTHMDB_QUERY_MODEL (tree_model);
 	guint index;
-	RhythmDBEntry *ret;
+	GSequencePtr ptr;
 
 	index = gtk_tree_path_get_indices (path)[0];
 
 	if (index >= g_sequence_get_length (model->priv->entries))
 		return FALSE;
 
-	ret = g_sequence_get_ptr_at_pos (model->priv->entries, index);
-	g_assert (ret);
+	ptr = g_sequence_get_ptr_at_pos (model->priv->entries, index);
+	g_assert (ptr);
 
 	iter->stamp = model->priv->stamp;
-	iter->user_data = ret;
+	iter->user_data = ptr;
 
 	return TRUE;
 }
