@@ -1,7 +1,7 @@
 /*
  *  arch-tag: Implementation of Rhythmbox tray icon object
  *
- *  Copyright (C) 2003 Colin Walters <walters@debian.org>
+ *  Copyright (C) 2003,2004 Colin Walters <walters@redhat.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,28 +25,25 @@
 #include <stdlib.h>
 #include <string.h>
 #include <libgnome/gnome-i18n.h>
-#include <bonobo/bonobo-ui-component.h>
-#include <bonobo/bonobo-ui-util.h>
-#include <bonobo/bonobo-main.h>
-#include <bonobo/bonobo-context.h>
-#include <bonobo/bonobo-exception.h>
-#include <bonobo/bonobo-window.h>
-#include <bonobo/bonobo-control-frame.h>
-#include <bonobo/bonobo-control.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 
 #include "disclosure-widget.h"
 #include "rb-tray-icon.h"
 #include "rb-stock-icons.h"
-#include "rb-bonobo-helpers.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
 #include "rb-preferences.h"
+#include "rb-shell.h"
 
 static void rb_tray_icon_class_init (RBTrayIconClass *klass);
 static void rb_tray_icon_init (RBTrayIcon *shell_player);
+static GObject *rb_tray_icon_constructor (GType type, guint n_construct_properties,
+					  GObjectConstructParam *construct_properties);
 static void rb_tray_icon_finalize (GObject *object);
+static void rb_tray_icon_sync_action (RBRemoteProxy *proxy, 
+				      gboolean visible, 
+				      RBTrayIcon *tray);
 static void rb_tray_icon_set_property (GObject *object,
 					  guint prop_id,
 					  const GValue *value,
@@ -55,17 +52,12 @@ static void rb_tray_icon_get_property (GObject *object,
 					  guint prop_id,
 					  GValue *value,
 					  GParamSpec *pspec);
-static void rb_tray_set_visibility (RBTrayIcon *tray, int state);
 static void rb_tray_icon_button_press_event_cb (GtkWidget *ebox, GdkEventButton *event,
 						RBTrayIcon *icon);
 static void rb_tray_icon_scroll_event_cb (GtkWidget *ebox, GdkEvent *event,
 						RBTrayIcon *icon);
-static void sync_menu (RBTrayIcon *tray);
-static void rb_tray_icon_show_window_changed_cb (BonoboUIComponent *component,
-					const char *path,
-					Bonobo_UIComponent_EventType type,
-					const char *state,
-					RBTrayIcon *icon);
+static void rb_tray_icon_show_window_changed_cb (GtkAction *action,
+						 RBTrayIcon *icon);
 static void rb_tray_icon_drop_cb (GtkWidget *widget,
 				  GdkDragContext *context,
 				  gint x,
@@ -75,46 +67,24 @@ static void rb_tray_icon_drop_cb (GtkWidget *widget,
 				  guint time,
 				  RBTrayIcon *icon);
 
-#define CMD_PATH_SHOW_WINDOW    "/commands/ShowWindow"
-
 struct RBTrayIconPrivate
 {
 	GtkTooltips *tooltips;
 
-	BonoboUIComponent *main_component;
-	BonoboUIComponent *tray_component;
+	GtkUIManager *ui_manager;
+	GtkActionGroup *actiongroup;
 
-	BonoboUIContainer *container;
-	BonoboControl *control;
-
-	GtkWindow *main_window;
 	GtkWidget *ebox;
 
-	RhythmDB *db;
-
-	int window_x;
-	int window_y;
-	int window_w;
-	int window_h;
-	gboolean visible;
+	RBRemoteProxy *proxy;
 };
 
 enum
 {
 	PROP_0,
-	PROP_CONTAINER,
-	PROP_COMPONENT,
-	PROP_DB,
-	PROP_TRAY_COMPONENT,
-	PROP_WINDOW,
-};
-
-enum
-{
-	VISIBILITY_HIDDEN,
-	VISIBILITY_VISIBLE,
-	VISIBILITY_SYNC,
-	VISIBILITY_TOGGLE
+	PROP_UI_MANAGER,
+	PROP_ACTION_GROUP,
+	PROP_REMOTE
 };
 
 enum
@@ -122,11 +92,13 @@ enum
 	LAST_SIGNAL,
 };
 
-static RBBonoboUIListener rb_tray_icon_listeners[] =
+static GtkToggleActionEntry rb_tray_icon_toggle_entries [] =
 {
-	RB_BONOBO_UI_LISTENER ("ShowWindow", (BonoboUIListenerFn) rb_tray_icon_show_window_changed_cb),
-	RB_BONOBO_UI_LISTENER_END
+	{ "TrayShowWindow", NULL, N_("_Show Window"), NULL,
+	  N_("Change the visibility of the main window"),
+	  G_CALLBACK (rb_tray_icon_show_window_changed_cb) }
 };
+static guint rb_tray_icon_n_toggle_entries = G_N_ELEMENTS (rb_tray_icon_toggle_entries);
 
 static const GtkTargetEntry target_uri [] = {{ "text/uri-list", 0, 0 }};
 
@@ -168,48 +140,32 @@ rb_tray_icon_class_init (RBTrayIconClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = rb_tray_icon_finalize;
+	object_class->constructor = rb_tray_icon_constructor;
 
 	object_class->set_property = rb_tray_icon_set_property;
 	object_class->get_property = rb_tray_icon_get_property;
 
 	g_object_class_install_property (object_class,
-					 PROP_WINDOW,
-					 g_param_spec_object ("window",
-							      "GtkWindow",
-							      "main GtkWindo",
-							      GTK_TYPE_WINDOW,
-							      G_PARAM_READWRITE));
-	g_object_class_install_property (object_class,
-					 PROP_CONTAINER,
-					 g_param_spec_object ("container",
-							      "BonoboUIContainer",
-							      "BonoboUIContainer object",
-							      BONOBO_TYPE_UI_CONTAINER,
+					 PROP_REMOTE,
+					 g_param_spec_object ("remote",
+							      "RBRemoteProxy",
+							      "RBRemoteProxy object",
+							      RB_TYPE_REMOTE_PROXY,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-
 	g_object_class_install_property (object_class,
-					 PROP_COMPONENT,
-					 g_param_spec_object ("component",
-							      "BonoboUIComponent",
-							      "BonoboUIComponent object",
-							      BONOBO_TYPE_UI_COMPONENT,
+					 PROP_UI_MANAGER,
+					 g_param_spec_object ("ui-manager",
+							      "GtkUIManager",
+							      "GtkUIManager object",
+							      GTK_TYPE_UI_MANAGER,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-
 	g_object_class_install_property (object_class,
-					 PROP_DB,
-					 g_param_spec_object ("db",
-							      "RhythmDB",
-							      "RhythmDB object",
-							      RHYTHMDB_TYPE,
+					 PROP_ACTION_GROUP,
+					 g_param_spec_object ("action-group",
+							      "GtkActionGroup",
+							      "GtkActionGroup object",
+							      GTK_TYPE_ACTION_GROUP,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-
-	g_object_class_install_property (object_class,
-					 PROP_TRAY_COMPONENT,
-					 g_param_spec_object ("tray_component",
-							      "BonoboUIComponent",
-							      "BonoboUIComponent object",
-							      BONOBO_TYPE_UI_COMPONENT,
-							      G_PARAM_READABLE));
 }
 
 static void
@@ -243,18 +199,25 @@ rb_tray_icon_init (RBTrayIcon *icon)
 					  GTK_ICON_SIZE_SMALL_TOOLBAR);
 	gtk_container_add (GTK_CONTAINER (icon->priv->ebox), image);
 	
-	icon->priv->control = bonobo_control_new (icon->priv->ebox);
-	icon->priv->tray_component = bonobo_control_get_popup_ui_component (icon->priv->control);
+	gtk_container_add (GTK_CONTAINER (icon), icon->priv->ebox);
+	gtk_widget_show_all (GTK_WIDGET (icon->priv->ebox));
+}
 
-	icon->priv->window_x = -1;
-	icon->priv->window_y = -1;
-	icon->priv->window_w = -1;
-	icon->priv->window_h = -1;
-	icon->priv->visible = TRUE;
+static GObject *
+rb_tray_icon_constructor (GType type, guint n_construct_properties,
+			  GObjectConstructParam *construct_properties)
+{
+	RBTrayIcon *tray;
+	RBTrayIconClass *klass;
+	GObjectClass *parent_class;  
 
-	rb_bonobo_add_listener_list_with_data (icon->priv->tray_component,
-						rb_tray_icon_listeners,
-						icon);
+	klass = RB_TRAY_ICON_CLASS (g_type_class_peek (RB_TYPE_TRAY_ICON));
+
+	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
+	tray = RB_TRAY_ICON (parent_class->constructor (type, n_construct_properties,
+							construct_properties));
+
+	return G_OBJECT (tray);
 }
 
 static void
@@ -276,35 +239,46 @@ rb_tray_icon_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+rb_tray_icon_sync_action (RBRemoteProxy *proxy, gboolean visible, RBTrayIcon *tray)
+{
+	GtkAction *action;
+	if ((tray->priv->actiongroup != NULL) && (tray->priv->proxy != NULL)) {
+		action = gtk_action_group_get_action (tray->priv->actiongroup,
+						      "TrayShowWindow");
+		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+					      rb_remote_proxy_get_visibility (tray->priv->proxy));
+	}
+}
+
 static void
 rb_tray_icon_set_property (GObject *object,
-			      guint prop_id,
-			      const GValue *value,
-			      GParamSpec *pspec)
+			   guint prop_id,
+			   const GValue *value,
+			   GParamSpec *pspec)
 {
 	RBTrayIcon *tray = RB_TRAY_ICON (object);
 
 	switch (prop_id)
 	{
-	case PROP_WINDOW:
-		tray->priv->main_window = g_value_get_object (value);
+	case PROP_REMOTE:
+		tray->priv->proxy = g_value_get_object (value);
+		g_signal_connect_object (G_OBJECT (tray->priv->proxy),
+					 "visibility_changed",
+					 G_CALLBACK (rb_tray_icon_sync_action),
+					 tray, 0);
+		rb_tray_icon_sync_action (NULL, FALSE, tray);
 		break;
-	case PROP_CONTAINER:
-	{
-		BonoboControlFrame *frame;
-		tray->priv->container = g_value_get_object (value);
-
-		frame = bonobo_control_frame_new (BONOBO_OBJREF (tray->priv->container));
-		bonobo_control_frame_bind_to_control (frame, BONOBO_OBJREF (tray->priv->control), NULL);
-		gtk_container_add (GTK_CONTAINER (tray), bonobo_control_frame_get_widget (frame));
-		gtk_widget_show_all (GTK_WIDGET (tray->priv->ebox));
+	case PROP_UI_MANAGER:
+		tray->priv->ui_manager = g_value_get_object (value);
 		break;
-	}
-	case PROP_COMPONENT:
-		tray->priv->main_component = g_value_get_object (value);
-		break;
-	case PROP_DB:
-		tray->priv->db = g_value_get_object (value);
+	case PROP_ACTION_GROUP:
+		tray->priv->actiongroup = g_value_get_object (value);
+		gtk_action_group_add_toggle_actions (tray->priv->actiongroup,
+						     rb_tray_icon_toggle_entries,
+						     rb_tray_icon_n_toggle_entries,
+						     tray);
+		rb_tray_icon_sync_action (NULL, FALSE, tray);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -322,20 +296,14 @@ rb_tray_icon_get_property (GObject *object,
 
 	switch (prop_id)
 	{
-	case PROP_WINDOW:
-		g_value_set_object (value, tray->priv->main_window);
+	case PROP_REMOTE:
+		g_value_set_object (value, tray->priv->proxy);
 		break;
-	case PROP_CONTAINER:
-		g_value_set_object (value, tray->priv->container);
+	case PROP_UI_MANAGER:
+		g_value_set_object (value, tray->priv->ui_manager);
 		break;
-	case PROP_COMPONENT:
-		g_value_set_object (value, tray->priv->main_component);
-		break;
-	case PROP_TRAY_COMPONENT:
-		g_value_set_object (value, tray->priv->tray_component);
-		break;
-	case PROP_DB:
-		g_value_set_object (value, tray->priv->db);
+	case PROP_ACTION_GROUP:
+		g_value_set_object (value, tray->priv->actiongroup);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -344,24 +312,16 @@ rb_tray_icon_get_property (GObject *object,
 }
 
 RBTrayIcon *
-rb_tray_icon_new (BonoboUIContainer *container,
-		  BonoboUIComponent *component,
-		  RhythmDB *db,
-		  GtkWindow *window)
+rb_tray_icon_new (GtkUIManager *mgr,
+		  GtkActionGroup *group,
+		  RBRemoteProxy *remote)
 {
-	RBTrayIcon *tray = g_object_new (RB_TYPE_TRAY_ICON,
-					 "title", "Rhythmbox tray icon",
-					 "container", container,
-					 "component", component,
-					 "db", db,
-					 "window", window,
-					 NULL);
-	
-	g_return_val_if_fail (tray->priv != NULL, NULL);
-
-	rb_tray_set_visibility (tray, VISIBILITY_SYNC);
-
-	return tray;
+	return g_object_new (RB_TYPE_TRAY_ICON,
+			     "title", "Rhythmbox tray icon",
+			     "ui-manager", mgr,
+			     "action-group", group,
+			     "remote", remote,
+			     NULL);
 }
 
 static void
@@ -376,16 +336,23 @@ rb_tray_icon_button_press_event_cb (GtkWidget *ebox, GdkEventButton *event,
 
 	switch (event->button) {
 	case 1:
-		rb_tray_set_visibility (icon, VISIBILITY_TOGGLE);
+	{
+		gboolean visible = rb_remote_proxy_get_visibility (icon->priv->proxy);
+		rb_remote_proxy_set_visibility (icon->priv->proxy, visible ? FALSE : TRUE);
 		break;
+	}
 
 	case 3:
-		/* contextmenu */
-		sync_menu (icon);
-		bonobo_control_do_popup (icon->priv->control, event->button,
-								event->time);
-		break;
-
+	{
+		GtkWidget *popup;
+		popup = gtk_ui_manager_get_widget (GTK_UI_MANAGER (icon->priv->ui_manager),
+						   "/RhythmboxTrayPopup");
+		gtk_menu_set_screen (GTK_MENU (popup), gtk_widget_get_screen (GTK_WIDGET (icon)));
+		gtk_menu_popup (GTK_MENU (popup), NULL, NULL,
+				NULL, NULL, 2,
+				gtk_get_current_event_time ());
+	}
+	break;
 	default:
 		break;
 	}
@@ -439,7 +406,7 @@ rb_tray_icon_drop_cb (GtkWidget *widget,
 	if (ret == FALSE)
 		return;
 
-	list = gnome_vfs_uri_list_parse (data->data);
+	list = gnome_vfs_uri_list_parse ((char *) data->data);
 
 	if (list == NULL) {
 		gtk_drag_finish (context, FALSE, FALSE, time);
@@ -460,9 +427,8 @@ rb_tray_icon_drop_cb (GtkWidget *widget,
 
 	for (i = uri_list; i != NULL; i = i->next) {
 		char *uri = i->data;
-
 		if (uri != NULL)
-			rhythmdb_add_uri_async (icon->priv->db, uri);
+			rb_remote_proxy_load_uri (icon->priv->proxy, uri, FALSE);
 
 		g_free (uri);
 	}
@@ -472,38 +438,13 @@ rb_tray_icon_drop_cb (GtkWidget *widget,
 	gtk_drag_finish (context, TRUE, FALSE, time);
 }
 
-
 static void
-sync_menu (RBTrayIcon *tray)
-{
-	BonoboUIComponent *pcomp;
-	BonoboUINode *node;
-
-	pcomp = bonobo_control_get_popup_ui_component (tray->priv->control);
-	
-	bonobo_ui_component_set (pcomp, "/", "<popups></popups>", NULL);
-
-	node = bonobo_ui_component_get_tree (tray->priv->main_component, "/popups/TrayPopup", TRUE, NULL);
-
-	bonobo_ui_node_set_attr (node, "name", "button3");
-
-	bonobo_ui_component_set_tree (pcomp, "/popups", node, NULL);
-	bonobo_ui_node_free (node);
-
-	node = bonobo_ui_component_get_tree (tray->priv->main_component, "/commands", TRUE, NULL);
-	bonobo_ui_component_set_tree (pcomp, "/", node, NULL);
-	bonobo_ui_node_free (node);
-}
-
-static void
-rb_tray_icon_show_window_changed_cb (BonoboUIComponent *component,
-				const char *path,
-				Bonobo_UIComponent_EventType type,
-				const char *state,
-				RBTrayIcon *icon)
+rb_tray_icon_show_window_changed_cb (GtkAction *action,
+				     RBTrayIcon *icon)
 {
 	rb_debug ("show window clicked");
-	rb_tray_set_visibility (icon, atoi (state));
+	rb_remote_proxy_set_visibility (icon->priv->proxy,
+					gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
 }
 
 void
@@ -514,58 +455,3 @@ rb_tray_icon_set_tooltip (RBTrayIcon *icon, const char *tooltip)
 			      tooltip, NULL);
 }
 
-static void
-rb_tray_restore_main_window (RBTrayIcon *icon)
-{
-	if ((icon->priv->window_x >= 0 && icon->priv->window_y >= 0) || (icon->priv->window_h >= 0 && icon->priv->window_w >=0 ))
-	{
-		gtk_widget_realize (GTK_WIDGET (icon->priv->main_window));
-		gdk_flush ();
-
-		if (icon->priv->window_x >= 0 && icon->priv->window_y >= 0)
-		{
-			gtk_window_move (icon->priv->main_window,
-					icon->priv->window_x,
-					icon->priv->window_y);                                }
-		if (icon->priv->window_w >= 0 && icon->priv->window_y >=0)
-		{
-			gtk_window_resize (icon->priv->main_window,
-					icon->priv->window_w,
-					icon->priv->window_h);
-		}
-	}
-}
-
-static void
-rb_tray_set_visibility (RBTrayIcon *icon, int state)
-{
-	switch (state)
-	{
-	case VISIBILITY_HIDDEN:
-       	case VISIBILITY_VISIBLE:
-		if (icon->priv->visible != state)
-			rb_tray_set_visibility (icon, VISIBILITY_TOGGLE);
-		break;
-        case VISIBILITY_TOGGLE:
-		icon->priv->visible = !icon->priv->visible;
-
-		if (icon->priv->visible == TRUE)
-		{
-			rb_tray_restore_main_window (icon);
-			gtk_widget_show (GTK_WIDGET (icon->priv->main_window));
-		} else {
-			gtk_window_get_position (icon->priv->main_window,
-						 &icon->priv->window_x,
-						 &icon->priv->window_y);
-			gtk_window_get_size (icon->priv->main_window,
-					     &icon->priv->window_w,
-					     &icon->priv->window_h);
-			gtk_widget_hide (GTK_WIDGET (icon->priv->main_window));
-		}
-
-	case VISIBILITY_SYNC:
-		rb_bonobo_set_active (icon->priv->main_component,
-				      CMD_PATH_SHOW_WINDOW,
-				      icon->priv->visible);
-	}
-}
