@@ -25,21 +25,18 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libxml/tree.h>
-#include <bonobo/bonobo-ui-component.h>
-#include <bonobo/bonobo-window.h>
 #include <string.h>
 
 #include "rb-iradio-source.h"
 
 #include "rhythmdb-query-model.h"
-#include "rb-bonobo-helpers.h"
 #include "rb-glade-helpers.h"
 #include "rb-stock-icons.h"
 #include "rb-entry-view.h"
 #include "rb-property-view.h"
 #include "rb-util.h"
 #include "rb-file-helpers.h"
-#include "rb-playlist.h"
+#include "totem-pl-parser.h"
 #include "rb-preferences.h"
 #include "rb-dialog.h"
 #include "rb-new-station-dialog.h"
@@ -59,6 +56,7 @@ static void rb_iradio_source_class_init (RBIRadioSourceClass *klass);
 static void rb_iradio_source_init (RBIRadioSource *source);
 static GObject *rb_iradio_source_constructor (GType type, guint n_construct_properties,
 					      GObjectConstructParam *construct_properties);
+static void rb_iradio_source_dispose (GObject *object);
 static void rb_iradio_source_finalize (GObject *object);
 static void rb_iradio_source_set_property (GObject *object,
 			                  guint prop_id,
@@ -69,7 +67,6 @@ static void rb_iradio_source_get_property (GObject *object,
 			                  GValue *value,
 			                  GParamSpec *pspec);
 static void rb_iradio_source_songs_show_popup_cb (RBEntryView *view,
-						  RhythmDBEntry *entry,
 						  RBIRadioSource *source);
 static void paned_size_allocate_cb (GtkWidget *widget,
 				    GtkAllocation *allocation,
@@ -87,10 +84,11 @@ static void rb_iradio_source_show_browser (RBIRadioSource *source,
 static void rb_iradio_source_state_prefs_sync (RBIRadioSource *source);
 static void genre_selected_cb (RBPropertyView *propview, const char *name,
 			       RBIRadioSource *iradio_source);
+static void genre_selection_reset_cb (RBPropertyView *propview, RBIRadioSource *iradio_source);
 static void rb_iradio_source_songs_view_sort_order_changed_cb (RBEntryView *view, RBIRadioSource *source);
 
 /* source methods */
-static const char *impl_get_status (RBSource *source);
+static char *impl_get_status (RBSource *source);
 static const char *impl_get_browser_key (RBSource *source);
 static const char *impl_get_search_key (RBSource *source);
 static GdkPixbuf *impl_get_pixbuf (RBSource *source);
@@ -99,16 +97,21 @@ static void impl_search (RBSource *source, const char *text);
 static void impl_delete (RBSource *source);
 static void impl_song_properties (RBSource *source);
 static RBSourceEOFType impl_handle_eos (RBSource *asource);
-static void impl_buffering_done (RBSource *asource);
+static gboolean impl_show_popup (RBSource *source);
 static void rb_iradio_source_do_query (RBIRadioSource *source, RBIRadioQueryType type);
 
 void rb_iradio_source_show_columns_changed_cb (GtkToggleButton *button,
 					     RBIRadioSource *source);
+static void stations_view_drag_data_received_cb (GtkWidget *widget,
+						 GdkDragContext *dc,
+						 gint x, gint y,
+						 GtkSelectionData *data,
+						 guint info, guint time,
+						 RBIRadioSource *source);
 
 #define CMD_PATH_SHOW_BROWSER "/commands/ShowBrowser"
 #define CMD_PATH_CURRENT_STATION "/commands/CurrentStation"
 #define CMD_PATH_SONG_INFO    "/commands/SongInfo"
-#define IRADIO_SOURCE_SONGS_POPUP_PATH "/popups/IRadioSongsList"
 
 #define CONF_UI_IRADIO_DIR CONF_PREFIX "/ui/iradio"
 #define CONF_UI_IRADIO_COLUMNS_SETUP CONF_PREFIX "/ui/iradio/columns_setup"
@@ -120,6 +123,8 @@ void rb_iradio_source_show_columns_changed_cb (GtkToggleButton *button,
 
 struct RBIRadioSourcePrivate
 {
+	gboolean disposed;
+	
 	RhythmDB *db;
 
 	GtkWidget *vbox;
@@ -129,14 +134,7 @@ struct RBIRadioSourcePrivate
 	RBPropertyView *genres;
 	RBEntryView *stations;
 
-	gboolean async_playcount_update_queued;
-	guint async_playcount_update_id;
-
-	char *title, *url, *song, *name;
-	int num_genres, num_stations;
-
-	gboolean shuffle;
-	gboolean repeat;
+	char *title, *song, *name;
 
 	gboolean initialized;
 
@@ -144,30 +142,23 @@ struct RBIRadioSourcePrivate
 
 	GtkWidget *paned;
 
-	gboolean lock;
-
-	gboolean loading_prefs;
-
-	RBIRadioQueryType query_type;
 	char *search_text;
 	char *selected_genre;
 
-	GtkWidget *genre_check;
-	GtkWidget *rating_check;
-	GtkWidget *play_count_check;
-	GtkWidget *last_played_check;
-	GtkWidget *quality_check;
+	RhythmDBEntryType entry_type;
 
-	guint async_idlenum;
-	gboolean async_entry_destroyed;
-	guint async_signum;
-	RhythmDB *async_update_entry;
+	RhythmDBQueryModel *all_query;
+};
+
+static const GtkTargetEntry stations_view_drag_types[] = {
+	{  "text/uri-list", 0, 0 },
+	{  "_NETSCAPE_URL", 0, 1 },
 };
 
 enum
 {
 	PROP_0,
-	PROP_DB,
+	PROP_ENTRY_TYPE
 };
 
 static GObjectClass *parent_class = NULL;
@@ -209,6 +200,7 @@ rb_iradio_source_class_init (RBIRadioSourceClass *klass)
 
 	parent_class = g_type_class_peek_parent (klass);
 
+	object_class->dispose = rb_iradio_source_dispose;
 	object_class->finalize = rb_iradio_source_finalize;
 	object_class->constructor = rb_iradio_source_constructor;
 
@@ -229,15 +221,17 @@ rb_iradio_source_class_init (RBIRadioSourceClass *klass)
 	source_class->impl_handle_eos = impl_handle_eos;
 	source_class->impl_have_artist_album = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_have_url = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_buffering_done = impl_buffering_done;
+	source_class->impl_show_popup = impl_show_popup;
 
 	g_object_class_install_property (object_class,
-					 PROP_DB,
-					 g_param_spec_object ("db",
-							      "RhythmDB",
-							      "RhythmDB database",
-							      RHYTHMDB_TYPE,
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+					 PROP_ENTRY_TYPE,
+					 g_param_spec_uint ("entry-type",
+							    "Entry type",
+							    "Type of the entries which should be displayed by this source",
+							    0,
+							    G_MAXINT,
+							    RHYTHMDB_ENTRY_TYPE_IRADIO_STATION,
+							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 	
 }
 
@@ -262,6 +256,20 @@ rb_iradio_source_init (RBIRadioSource *source)
 	gtk_widget_destroy (dummy);
 }
 
+static void
+rb_iradio_source_dispose (GObject *object)
+{
+	RBIRadioSource *source;
+
+	source = RB_IRADIO_SOURCE (object);
+	if (source->priv->disposed)
+		return;
+	source->priv->disposed = TRUE;
+	
+	g_object_unref (source->priv->all_query);
+	g_object_unref (source->priv->db);
+}
+
 static void
 rb_iradio_source_finalize (GObject *object)
 {
@@ -293,6 +301,8 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 	RBIRadioSource *source;
 	RBIRadioSourceClass *klass;
 	GObjectClass *parent_class;  
+	RBShell *shell;
+
 	klass = RB_IRADIO_SOURCE_CLASS (g_type_class_peek (RB_TYPE_IRADIO_SOURCE));
 
 	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
@@ -301,35 +311,57 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 
 	source->priv->paned = gtk_hpaned_new ();
 
+	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
+	g_object_get (G_OBJECT (shell), "db", &source->priv->db, NULL);
+	g_object_unref (G_OBJECT (shell));
+
+
 	/* set up stations view */
 	source->priv->stations = rb_entry_view_new (source->priv->db, CONF_STATE_IRADIO_SORTING,
 						    FALSE, FALSE);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_TITLE);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_GENRE);
-	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_QUALITY);
+/* 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_QUALITY); */
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_RATING);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_PLAY_COUNT);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_LAST_PLAYED);
-	g_signal_connect (G_OBJECT (source->priv->stations),
-			  "sort-order-changed",
-			  G_CALLBACK (rb_iradio_source_songs_view_sort_order_changed_cb),
-			  source);
-
-	g_signal_connect (G_OBJECT (source->priv->stations),
-			  "size_allocate",
-			  G_CALLBACK (paned_size_allocate_cb),
-			  source);
-	g_signal_connect (G_OBJECT (source->priv->stations), "show_popup",
-			  G_CALLBACK (rb_iradio_source_songs_show_popup_cb), source);
+	g_signal_connect_object (G_OBJECT (source->priv->stations),
+				 "sort-order-changed",
+				 G_CALLBACK (rb_iradio_source_songs_view_sort_order_changed_cb),
+				 source, 0);
+	
+	/* set up drag and drop for the song tree view.
+	 * we don't use RBEntryView's DnD support because it does too much.
+	 * we just want to be able to drop stations in to add them.
+	 */
+	g_signal_connect_object (G_OBJECT (source->priv->stations), 
+				 "drag_data_received",
+				 G_CALLBACK (stations_view_drag_data_received_cb),
+				 source, 0);
+	gtk_drag_dest_set (GTK_WIDGET (source->priv->stations),
+			   GTK_DEST_DEFAULT_ALL,
+			   stations_view_drag_types, 2,
+			   GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
+	g_signal_connect_object (G_OBJECT (source->priv->stations),
+				 "size_allocate",
+				 G_CALLBACK (paned_size_allocate_cb),
+				 source, 0);
+	g_signal_connect_object (G_OBJECT (source->priv->stations), "show_popup",
+				 G_CALLBACK (rb_iradio_source_songs_show_popup_cb), source, 0);
+	
 	/* set up genre entry view */
 	source->priv->genres = rb_property_view_new (source->priv->db,
 						     RHYTHMDB_PROP_GENRE,
 						     _("Genre"));
-	g_signal_connect (G_OBJECT (source->priv->genres),
-			  "property-selected",
-			  G_CALLBACK (genre_selected_cb),
-			  source);
+	g_signal_connect_object (G_OBJECT (source->priv->genres),
+				 "property-selected",
+				 G_CALLBACK (genre_selected_cb),
+				 source, 0);
+	g_signal_connect_object (G_OBJECT (source->priv->genres),
+				 "property-selection-reset",
+				 G_CALLBACK (genre_selection_reset_cb),
+				 source, 0);
 
 	g_object_set (G_OBJECT (source->priv->genres), "vscrollbar_policy",
 		      GTK_POLICY_AUTOMATIC, NULL);
@@ -365,8 +397,8 @@ rb_iradio_source_set_property (GObject *object,
 
 	switch (prop_id)
 	{
-	case PROP_DB:
-		source->priv->db = g_value_get_object (value);
+	case PROP_ENTRY_TYPE:
+		source->priv->entry_type = g_value_get_uint (value);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -384,8 +416,8 @@ rb_iradio_source_get_property (GObject *object,
 
 	switch (prop_id)
 	{
-	case PROP_DB:
-		g_value_set_object (value, source->priv->db);
+	case PROP_ENTRY_TYPE:
+		g_value_set_uint (value, source->priv->entry_type);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -394,16 +426,14 @@ rb_iradio_source_get_property (GObject *object,
 }
 
 RBSource *
-rb_iradio_source_new (RBShell *shell, RhythmDB *db, 
-		      BonoboUIComponent *component)
+rb_iradio_source_new (RBShell *shell)
 {
 	RBSource *source;
 
 	source = RB_SOURCE (g_object_new (RB_TYPE_IRADIO_SOURCE,
 					  "name", _("Radio"),
+					  "shell", shell,
 					  "internal-name", "<radio>",
-					  "db", db,
-					  "component", component,
 					  NULL));
 
 	rb_shell_register_entry_type_for_source (shell, source, 
@@ -419,7 +449,18 @@ rb_iradio_source_add_station (RBIRadioSource *source,
 	RhythmDBEntry *entry;
 	GValue val = { 0, };
 
-	rhythmdb_write_lock (source->priv->db);
+	if (uri && !g_utf8_validate (uri, -1, NULL)) {
+		rb_debug ("uri \"%s\" is not valid UTF-8", uri);
+		return;
+	}
+	if (title && !g_utf8_validate (title, -1, NULL)) {
+		rb_debug ("title \"s\" is not valid UTF-8", title);
+		return;
+	}
+	if (genre && !g_utf8_validate (genre, -1, NULL)) {
+		rb_debug ("genre \"s\" is not valid UTF-8", genre);
+		genre = "";
+	}
 
 	entry = rhythmdb_entry_lookup_by_location (source->priv->db, uri);
 	if (!entry)
@@ -433,10 +474,10 @@ rb_iradio_source_add_station (RBIRadioSource *source,
 	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_TITLE, &val);
 	g_value_reset (&val);
 		
-	if (genre) {
-		g_value_set_string (&val, genre);
-		rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_GENRE, &val);
-	}
+	if ((!genre) || (strcmp (genre, "") == 0))
+		genre = _("Unknown");
+	g_value_set_string (&val, genre);
+	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_GENRE, &val);
 	g_value_unset (&val);
 	
 	g_value_init (&val, G_TYPE_DOUBLE);
@@ -444,7 +485,7 @@ rb_iradio_source_add_station (RBIRadioSource *source,
 	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_RATING, &val);
 	g_value_unset (&val);
 	
-	rhythmdb_write_unlock (source->priv->db);
+	rhythmdb_commit (source->priv->db);
 }
 
 static GdkPixbuf *
@@ -497,6 +538,7 @@ impl_handle_eos (RBSource *asource)
 	return RB_SOURCE_EOF_ERROR;
 }
 
+#if 0
 struct RBIRadioAsyncPlayStatisticsData
 {
 	RBIRadioSource *source;
@@ -527,30 +569,9 @@ rb_iradio_source_async_update_play_statistics (gpointer data)
 
 	return FALSE;
 }
+#endif
 
-static void
-impl_buffering_done (RBSource *asource)
-{
-	RBIRadioSource *source = RB_IRADIO_SOURCE (asource);
-	RhythmDBEntry *entry = rb_entry_view_get_playing_entry (source->priv->stations);
-
-	rb_debug ("queueing async play statistics update, entry: %p", entry);
-
-	if (source->priv->async_update_entry != NULL) {
-		rb_debug ("async handler already queued, removing");
-		rhythmdb_entry_unref (source->priv->db, source->priv->async_update_entry);
-		g_source_remove (source->priv->async_idlenum);
-	}
-
-	source->priv->async_entry_destroyed = FALSE;
-	source->priv->async_update_entry = entry;
-	rhythmdb_entry_ref (source->priv->db, source->priv->async_update_entry);
-	source->priv->async_idlenum =
-		g_timeout_add (6000, (GSourceFunc) rb_iradio_source_async_update_play_statistics,
-			       source);
-}
-
-static const char *
+static char *
 impl_get_status (RBSource *asource)
 {
  	RBIRadioSource *source = RB_IRADIO_SOURCE (asource);
@@ -581,9 +602,8 @@ impl_delete (RBSource *asource)
 
 	for (l = rb_entry_view_get_selected_entries (source->priv->stations); l != NULL;
 	     l = g_list_next (l)) {
-		rhythmdb_write_lock (source->priv->db);
 		rhythmdb_entry_delete (source->priv->db, l->data);
-		rhythmdb_write_unlock (source->priv->db);
+		rhythmdb_commit (source->priv->db);
 	}
 }
 
@@ -641,24 +661,12 @@ rb_iradio_source_songs_view_sort_order_changed_cb (RBEntryView *view,
 
 static void
 rb_iradio_source_songs_show_popup_cb (RBEntryView *view,
-				      RhythmDBEntry *entry,
 				      RBIRadioSource *source)
 {
-	GtkWidget *menu;
-	GtkWidget *window;
-
-	window = gtk_widget_get_ancestor (GTK_WIDGET (view),
-					  BONOBO_TYPE_WINDOW);
-
-	menu = gtk_menu_new ();
-
-	bonobo_window_add_popup (BONOBO_WINDOW (window), GTK_MENU (menu),
-			         IRADIO_SOURCE_SONGS_POPUP_PATH);
-
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
-			3, gtk_get_current_event_time ());
-
-	gtk_object_sink (GTK_OBJECT (menu));
+	if (G_OBJECT (source) == NULL) {
+		return;
+	}
+	_rb_source_show_popup (RB_SOURCE (source), "/IRadioViewPopup");
 }
 
 static void
@@ -672,6 +680,17 @@ genre_selected_cb (RBPropertyView *propview, const char *name,
 	rb_source_notify_filter_changed (RB_SOURCE (iradio_source));
 }
 
+static void
+genre_selection_reset_cb (RBPropertyView *propview, RBIRadioSource *iradio_source)
+{
+	g_free (iradio_source->priv->selected_genre);
+	iradio_source->priv->selected_genre = NULL;
+
+	rb_iradio_source_do_query (iradio_source, RB_IRADIO_QUERY_TYPE_GENRE);
+
+	rb_source_notify_filter_changed (RB_SOURCE (iradio_source));
+}
+
 static void
 rb_iradio_source_show_browser (RBIRadioSource *source,
 			       gboolean show)
@@ -693,84 +712,89 @@ rb_iradio_source_show_browser (RBIRadioSource *source,
 static void
 rb_iradio_source_do_query (RBIRadioSource *source, RBIRadioQueryType qtype)
 {
+	RhythmDBEntryType entry_type;
 	RhythmDBQueryModel *query_model;
 	RhythmDBPropertyModel *genre_model;
 	GtkTreeModel *model;
 	GPtrArray *query;
 
-	rhythmdb_read_lock (source->priv->db);
-	source->priv->query_type = qtype;
+	if ((qtype == RB_IRADIO_QUERY_TYPE_ALL) && (source->priv->all_query)) {
+		rb_debug ("using cached query");
+		rb_entry_view_set_model (source->priv->stations, RHYTHMDB_QUERY_MODEL (source->priv->all_query));
+		return;
+	}
+
+	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
 	query = rhythmdb_query_parse (source->priv->db,
 				      RHYTHMDB_QUERY_PROP_EQUALS,
 				      RHYTHMDB_PROP_TYPE,
-				      RHYTHMDB_ENTRY_TYPE_IRADIO_STATION,
+				      entry_type,
 				      RHYTHMDB_QUERY_END);
-
-	if (source->priv->search_text) {
-		GPtrArray *subquery = rhythmdb_query_parse (source->priv->db,
-							    RHYTHMDB_QUERY_PROP_LIKE,
-							    RHYTHMDB_PROP_GENRE_FOLDED,
-							    source->priv->search_text,
-							    RHYTHMDB_QUERY_DISJUNCTION,
-							    RHYTHMDB_QUERY_PROP_LIKE,
-							    RHYTHMDB_PROP_TITLE_FOLDED,
-							    source->priv->search_text,
-							    RHYTHMDB_QUERY_END);
-		rhythmdb_query_append (source->priv->db,
-				       query,
-				       RHYTHMDB_QUERY_SUBQUERY,
-				       subquery,
-				       RHYTHMDB_QUERY_END);
-	}
-
 	query_model = rhythmdb_query_model_new_empty (source->priv->db);
 
-	if (qtype < RB_IRADIO_QUERY_TYPE_GENRE) {
+	if (qtype == RB_IRADIO_QUERY_TYPE_ALL) {
+		rb_debug ("building new all query");
 		rb_property_view_reset (source->priv->genres);
 		g_free (source->priv->selected_genre);
 		source->priv->selected_genre = NULL;
-	} 
 
-	genre_model = rb_property_view_get_model (source->priv->genres);
-
-	if (qtype < RB_IRADIO_QUERY_TYPE_GENRE) {
+		genre_model = rb_property_view_get_model (source->priv->genres);
 		g_object_set (G_OBJECT (genre_model), "query-model", query_model, NULL);
-	} else
-		g_object_set (G_OBJECT (genre_model), "query-model", NULL, NULL);
 
-	if (source->priv->selected_genre)
-		rhythmdb_query_append (source->priv->db,
-				       query,
-				       RHYTHMDB_QUERY_PROP_EQUALS,
-				       RHYTHMDB_PROP_GENRE,
-				       source->priv->selected_genre,
-				       RHYTHMDB_QUERY_END);
+		source->priv->all_query = query_model;
+	} else {
+		if (source->priv->search_text) {
+			GPtrArray *subquery = rhythmdb_query_parse (source->priv->db,
+								    RHYTHMDB_QUERY_PROP_LIKE,
+								    RHYTHMDB_PROP_GENRE_FOLDED,
+								    source->priv->search_text,
+								    RHYTHMDB_QUERY_DISJUNCTION,
+								    RHYTHMDB_QUERY_PROP_LIKE,
+								    RHYTHMDB_PROP_TITLE_FOLDED,
+								    source->priv->search_text,
+								    RHYTHMDB_QUERY_END);
+			rhythmdb_query_append (source->priv->db,
+					       query,
+					       RHYTHMDB_QUERY_SUBQUERY,
+					       subquery,
+					       RHYTHMDB_QUERY_END);
+		}
+
+
+		if (source->priv->selected_genre)
+			rhythmdb_query_append (source->priv->db,
+					       query,
+					       RHYTHMDB_QUERY_PROP_EQUALS,
+					       RHYTHMDB_PROP_GENRE,
+					       source->priv->selected_genre,
+					       RHYTHMDB_QUERY_END);
+	}
+
 
 	model = GTK_TREE_MODEL (query_model);
 	
 	rb_entry_view_set_model (source->priv->stations, RHYTHMDB_QUERY_MODEL (query_model));
 
-	rhythmdb_do_full_query_parsed (source->priv->db, model, query);
+	/* we rely on getting signals from all_query to update the genre view.
+	 * we keep it marked as connected so it doesn't just eat the signals.
+	 */
+	if (query_model != source->priv->all_query)
+		rhythmdb_query_model_set_connected (source->priv->all_query, TRUE);
 
-	if (qtype >= RB_IRADIO_QUERY_TYPE_GENRE)
-		g_object_set (G_OBJECT (genre_model), "query-model", query_model, NULL);
+	rhythmdb_do_full_query_parsed (source->priv->db, model, query);
 
 	rhythmdb_query_free (query);
 
-	rhythmdb_read_unlock (source->priv->db);
 }
 
 static void
-handle_playlist_entry_cb (RBPlaylist *playlist, const char *uri, const char *title,
+handle_playlist_entry_cb (TotemPlParser *playlist, const char *uri, const char *title,
 			  const char *genre, RBIRadioSource *source)
 {
 	if (rb_uri_is_iradio (uri)) {
-		rhythmdb_read_lock (source->priv->db);
 		if (!rhythmdb_entry_lookup_by_location (source->priv->db, uri)) {
-			rhythmdb_read_unlock (source->priv->db);
 			rb_iradio_source_add_station (source, uri, title, genre);
-		} else
-			rhythmdb_read_unlock (source->priv->db);
+		}
 	}
 }
 
@@ -780,12 +804,76 @@ rb_iradio_source_first_time_changed (GConfClient *client,
 				     GConfEntry *entry,
 				     RBIRadioSource *source)
 {
-	RBPlaylist *parser = rb_playlist_new ();
+	TotemPlParser *parser = totem_pl_parser_new ();
 
 	g_signal_connect_object (G_OBJECT (parser), "entry",
 				 G_CALLBACK (handle_playlist_entry_cb),
 				 source, 0);
-	rb_playlist_parse (parser, rb_file ("iradio-initial.pls"));	
+	totem_pl_parser_parse (parser, rb_file ("iradio-initial.pls"), FALSE);	
 	g_object_unref (G_OBJECT (parser));
 }
 
+static void
+stations_view_drag_data_received_cb (GtkWidget *widget,
+				     GdkDragContext *dc,
+				     gint x, gint y,
+				     GtkSelectionData *selection_data, 
+				     guint info, guint time,
+				     RBIRadioSource *source)
+{
+	GList *list, *uri_list, *i;
+
+	rb_debug ("parsing uri list");
+	list = gnome_vfs_uri_list_parse ((char *)selection_data->data);
+
+	if (list == NULL)
+		return;
+
+	uri_list = NULL;
+
+	for (i = list; i != NULL; i = g_list_next (i))
+		uri_list = g_list_append (uri_list, gnome_vfs_uri_to_string ((const GnomeVFSURI *) i->data, 0));
+
+	gnome_vfs_uri_list_free (list);
+
+	if (uri_list == NULL)
+		return;
+
+	rb_debug ("adding uris");
+
+	for (i = uri_list; i != NULL; i = i->next) {
+		char *uri = NULL;
+		char *title_data = NULL;
+		char *title = NULL;
+
+		uri = i->data;
+
+		/* as totem source says, "Super _NETSCAPE_URL trick" */
+		if (info == 1) {
+			i = i->next;
+			if (i != NULL) {
+				g_free (i->data);
+			}
+		}
+
+		if ((uri != NULL) && 
+		    (rb_uri_is_iradio (uri)) &&
+		    (!rhythmdb_entry_lookup_by_location (source->priv->db, uri))) {
+			rb_iradio_source_add_station (source, uri, title, NULL);
+		}
+
+		g_free (uri);
+		g_free (title_data);
+	}
+
+	g_list_free (uri_list);
+	return;
+}
+
+static gboolean
+impl_show_popup (RBSource *source)
+{
+	_rb_source_show_popup (RB_SOURCE (source), "/IRadioSourcePopup");
+	return TRUE;
+}
+
