@@ -23,8 +23,9 @@
 #define __RB_LIBRARY_SOURCE_H
 
 #include <gtk/gtkwindow.h>
-#include <bonobo/bonobo-ui-container.h>
+#include <gtk/gtkactiongroup.h>
 
+#include "rb-shell.h"
 #include "rb-source.h"
 #include "rhythmdb.h"
 
@@ -53,12 +54,16 @@ typedef struct
 
 GType		rb_library_source_get_type	(void);
 
-RBSource *	rb_library_source_new		(RhythmDB *db);
+RBSource *      rb_library_source_new           (RBShell *shell);
+
 void		rb_library_source_show_browser	(RBLibrarySource *source,
 						 gboolean show);
 
 void		rb_library_source_add_location	(RBLibrarySource *source, GtkWindow *win);
 
+void            rb_library_source_class_add_actions (RBShell *shell, 
+						     GtkActionGroup *uimgr);
+
 G_END_DECLS
 
 #endif /* __RB_LIBRARY_SOURCE_H */
