@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  arch-tag: Implementation of playlist source object
  *
@@ -24,9 +25,8 @@
 #include <gtk/gtk.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
+#include <totem-pl-parser.h>
 #include <libxml/tree.h>
-#include <bonobo/bonobo-ui-component.h>
-#include <bonobo/bonobo-window.h>
 #include <unistd.h>
 #include <string.h>
 
@@ -34,24 +34,36 @@
 #include "rb-entry-view.h"
 #include "rb-search-entry.h"
 #include "rb-file-helpers.h"
-#include "rb-playlist.h"
 #include "rb-preferences.h"
 #include "rb-dialog.h"
 #include "rb-util.h"
 #include "rb-playlist-source.h"
+#if defined(WITH_CD_BURNER_SUPPORT)
+#include "rb-playlist-source-recorder.h"
+#endif
 #include "rb-volume.h"
-#include "rb-bonobo-helpers.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
 #include "rb-song-info.h"
-#include "rb-tree-view-column.h"
 
 #define RB_PLAYLIST_XML_VERSION "1.0"
 
+#define RB_PLAYLIST_PLAYLIST (xmlChar *) "playlist"
+#define RB_PLAYLIST_TYPE (xmlChar *) "type"
+#define RB_PLAYLIST_AUTOMATIC (xmlChar *) "automatic"
+#define RB_PLAYLIST_STATIC (xmlChar *) "static"
+#define RB_PLAYLIST_NAME (xmlChar *) "name"
+#define RB_PLAYLIST_INT_NAME (xmlChar *) "internal-name"
+#define RB_PLAYLIST_LIMIT_COUNT (xmlChar *) "limit-count"
+#define RB_PLAYLIST_LIMIT_SIZE (xmlChar *) "limit-size"
+#define RB_PLAYLIST_LIMIT (xmlChar *) "limit"
+#define RB_PLAYLIST_LOCATION (xmlChar *) "location"
+
 static void rb_playlist_source_class_init (RBPlaylistSourceClass *klass);
 static void rb_playlist_source_init (RBPlaylistSource *source);
 static GObject *rb_playlist_source_constructor (GType type, guint n_construct_properties,
 						GObjectConstructParam *construct_properties);
+static void rb_playlist_source_dispose (GObject *object);
 static void rb_playlist_source_finalize (GObject *object);
 static void rb_playlist_source_set_property (GObject *object,
 			                  guint prop_id,
@@ -63,7 +75,7 @@ static void rb_playlist_source_get_property (GObject *object,
 			                  GParamSpec *pspec);
 
 /* source methods */
-static const char *impl_get_status (RBSource *source);
+static char *impl_get_status (RBSource *source);
 static const char *impl_get_browser_key (RBSource *source);
 static GdkPixbuf *impl_get_pixbuf (RBSource *source);
 static RBEntryView *impl_get_entry_view (RBSource *source);
@@ -89,11 +101,14 @@ static void rb_playlist_source_drop_cb (GtkWidget *widget,
 static void rb_playlist_source_add_list_uri (RBPlaylistSource *source,
 					  GList *list);
 
-#define PLAYLIST_SOURCE_SONGS_POPUP_PATH "/popups/PlaylistSongsList"
-#define PLAYLIST_SOURCE_POPUP_PATH "/popups/PlaylistSourceList"
+#define PLAYLIST_SOURCE_SONGS_POPUP_PATH "/PlaylistViewPopup"
+#define PLAYLIST_SOURCE_POPUP_PATH "/PlaylistSourcePopup"
+#define PLAYLIST_SOURCE_AUTOMATIC_POPUP_PATH "/SmartPlaylistSourcePopup"
 
 struct RBPlaylistSourcePrivate
 {
+	gboolean disposed;
+	
 	RhythmDB *db;
 
 	gboolean automatic;
@@ -107,16 +122,16 @@ struct RBPlaylistSourcePrivate
 
 	RBEntryView *songs;
 
+	gboolean dirty;
+
 	char *title;
-	char *status;
 };
 
 enum
 {
 	PROP_0,
-	PROP_LIBRARY,
-	PROP_DB,
 	PROP_AUTOMATIC,
+	PROP_DIRTY
 };
 
 static GObjectClass *parent_class = NULL;
@@ -159,6 +174,7 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 
 	parent_class = g_type_class_peek_parent (klass);
 
+	object_class->dispose = rb_playlist_source_dispose;
 	object_class->finalize = rb_playlist_source_finalize;
 	object_class->constructor = rb_playlist_source_constructor;
 
@@ -184,14 +200,6 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 	source_class->impl_receive_drag = impl_receive_drag;
 	source_class->impl_show_popup = impl_show_popup;
 
-	g_object_class_install_property (object_class,
-					 PROP_DB,
-					 g_param_spec_object ("db",
-							      "RhythmDB",
-							      "RhythmDB database",
-							      RHYTHMDB_TYPE,
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-
 	g_object_class_install_property (object_class,
 					 PROP_AUTOMATIC,
 					 g_param_spec_boolean ("automatic",
@@ -199,6 +207,14 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 							       "whether this playlist is a smartypants",
 							       FALSE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property (object_class,
+					 PROP_DIRTY,
+					 g_param_spec_boolean ("dirty",
+							       "dirty",
+							       "whether this playlist should be saved",
+							       FALSE,
+							       G_PARAM_READABLE));
 }
 
 static void
@@ -229,19 +245,24 @@ rb_playlist_source_constructor (GType type, guint n_construct_properties,
 	RBPlaylistSourceClass *klass;
 	GObjectClass *parent_class;  
 	GtkWidget *dummy = gtk_tree_view_new ();
+	RBShell *shell;
 
-	klass = RB_PLAYLIST_SOURCE_CLASS (g_type_class_peek (type));
+	klass = RB_PLAYLIST_SOURCE_CLASS (g_type_class_peek (RB_TYPE_PLAYLIST_SOURCE));
 
 	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
 	source = RB_PLAYLIST_SOURCE (parent_class->constructor (type, n_construct_properties,
 								construct_properties));
 
-	g_signal_connect (G_OBJECT (source->priv->db), "entry_added",
-			  G_CALLBACK (rb_playlist_source_entry_added_cb),
-			  source);
-	g_signal_connect (G_OBJECT (source->priv->db), "entry_restored",
-			  G_CALLBACK (rb_playlist_source_entry_added_cb),
-			  source);
+	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
+	g_object_get (RB_SHELL (shell), "db", &source->priv->db, NULL);
+	g_object_unref (G_OBJECT (shell));
+
+	g_signal_connect_object (G_OBJECT (source->priv->db), "entry_added",
+				 G_CALLBACK (rb_playlist_source_entry_added_cb),
+				 source, 0);
+	g_signal_connect_object (G_OBJECT (source->priv->db), "entry_restored",
+				 G_CALLBACK (rb_playlist_source_entry_added_cb),
+				 source, 0);
 
 	source->priv->vbox = gtk_vbox_new (FALSE, 5);
 
@@ -256,7 +277,7 @@ rb_playlist_source_constructor (GType type, guint n_construct_properties,
 	rb_entry_view_set_model (source->priv->songs, RHYTHMDB_QUERY_MODEL (source->priv->model));
 
 	{
-		GtkTreeViewColumn *column = GTK_TREE_VIEW_COLUMN (rb_tree_view_column_new ());
+		GtkTreeViewColumn *column = gtk_tree_view_column_new ();
 		GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
 		gtk_tree_view_column_pack_start (column, renderer, TRUE);
 
@@ -279,11 +300,11 @@ rb_playlist_source_constructor (GType type, guint n_construct_properties,
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_LAST_PLAYED);
 	rb_entry_view_set_columns_clickable (source->priv->songs, FALSE);
 
-	g_signal_connect (G_OBJECT (source->priv->songs), "show_popup",
-			  G_CALLBACK (rb_playlist_source_songs_show_popup_cb), source);
+	g_signal_connect_object (G_OBJECT (source->priv->songs), "show_popup",
+				 G_CALLBACK (rb_playlist_source_songs_show_popup_cb), source, 0);
 		
-	g_signal_connect (G_OBJECT (source->priv->songs), "drag_data_received",
-			  G_CALLBACK (rb_playlist_source_drop_cb), source);
+	g_signal_connect_object (G_OBJECT (source->priv->songs), "drag_data_received",
+				 G_CALLBACK (rb_playlist_source_drop_cb), source, 0);
 	gtk_drag_dest_set (GTK_WIDGET (source->priv->songs), GTK_DEST_DEFAULT_ALL,
 			   target_uri, G_N_ELEMENTS (target_uri), GDK_ACTION_COPY);
 		
@@ -308,7 +329,8 @@ static void
 rb_playlist_source_songs_show_popup_cb (RBEntryView *view,
 					RBPlaylistSource *playlist_view)
 {
-	rb_bonobo_show_popup (GTK_WIDGET (view), PLAYLIST_SOURCE_SONGS_POPUP_PATH);
+	_rb_source_show_popup (RB_SOURCE (playlist_view), 
+			       PLAYLIST_SOURCE_SONGS_POPUP_PATH);
 }
 
 static void
@@ -317,6 +339,19 @@ rb_playlist_source_init (RBPlaylistSource *source)
 	source->priv = g_new0 (RBPlaylistSourcePrivate, 1);
 }
 
+static void
+rb_playlist_source_dispose (GObject *object)
+{
+	RBPlaylistSource *source;
+	source = RB_PLAYLIST_SOURCE (object);
+
+	if (source->priv->disposed)
+		return;
+	source->priv->disposed = TRUE;
+
+	g_object_unref (source->priv->db);
+}
+
 static void
 rb_playlist_source_finalize (GObject *object)
 {
@@ -332,7 +367,6 @@ rb_playlist_source_finalize (GObject *object)
 	g_hash_table_destroy (source->priv->entries);
 
 	g_free (source->priv->title);
-	g_free (source->priv->status);
 
 	g_free (source->priv);
 
@@ -349,9 +383,6 @@ rb_playlist_source_set_property (GObject *object,
 
 	switch (prop_id)
 	{
-	case PROP_DB:
-		source->priv->db = g_value_get_object (value);
-		break;
 	case PROP_AUTOMATIC:
 		source->priv->automatic = g_value_get_boolean (value);
 		break;
@@ -371,12 +402,13 @@ rb_playlist_source_get_property (GObject *object,
 
 	switch (prop_id)
 	{
-	case PROP_DB:
-		g_value_set_object (value, source->priv->db);
-		break;
 	case PROP_AUTOMATIC:
 		g_value_set_boolean (value, source->priv->automatic);
 		break;
+	case PROP_DIRTY:
+		g_value_set_boolean (value, source->priv->dirty);
+		break;
+
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -384,13 +416,13 @@ rb_playlist_source_get_property (GObject *object,
 }
 
 RBSource *
-rb_playlist_source_new (RhythmDB *db, gboolean automatic)
+rb_playlist_source_new (RBShell *shell, gboolean automatic)
 {
 	RBSource *source;
 	
 	source = RB_SOURCE (g_object_new (RB_TYPE_PLAYLIST_SOURCE,
 					  "name", _("Unknown"),
-					  "db", db,
+					  "shell", shell,
 					  "automatic", automatic,
 					  NULL));
 
@@ -406,10 +438,10 @@ rb_playlist_source_entry_added_cb (RhythmDB *db, RhythmDBEntry *entry,
 	if (source->priv->automatic)
 		return;
 	
-	location = rhythmdb_entry_get_string (db, entry,
-					      RHYTHMDB_PROP_LOCATION);
+	location = entry->location;
 	if (g_hash_table_lookup (source->priv->entries, location)) {
 		rhythmdb_query_model_add_entry (source->priv->model, entry);
+		source->priv->dirty = TRUE;
 	}
 }
 
@@ -441,23 +473,37 @@ rb_playlist_source_set_query (RBPlaylistSource *source,
 
 	rb_entry_view_set_model (source->priv->songs, RHYTHMDB_QUERY_MODEL (query_model));
 
-	rhythmdb_read_lock (source->priv->db);
 	rhythmdb_do_full_query_async_parsed (source->priv->db, model, query);
 
 	g_object_unref (G_OBJECT (query_model));
 	rb_entry_view_poll_model (source->priv->songs);
 }
 
-static const char *
+void
+rb_playlist_source_get_query (RBPlaylistSource *source,
+			      GPtrArray **query,
+			      guint *limit_count,
+			      guint *limit_mb)
+{
+	g_assert (source->priv->automatic);
+
+	g_object_get (G_OBJECT (source->priv->model),
+		      "query", query,
+		      "max-count", limit_count,
+		      "max-size", limit_mb, NULL);
+}
+
+static char *
 impl_get_status (RBSource *asource)
 {
 
 	RBPlaylistSource *source = RB_PLAYLIST_SOURCE (asource);
-	g_free (source->priv->status);
-	source->priv->status = rhythmdb_compute_status_normal (rb_entry_view_get_num_entries (source->priv->songs),
-							       rb_entry_view_get_duration (source->priv->songs),
-							       rb_entry_view_get_total_size (source->priv->songs));
-	return source->priv->status;
+	gchar *status;
+
+	status = rhythmdb_compute_status_normal (rb_entry_view_get_num_entries (source->priv->songs),
+						 rb_entry_view_get_duration (source->priv->songs),
+						 rb_entry_view_get_total_size (source->priv->songs));
+	return status;
 }
 
 static const char *
@@ -537,25 +583,67 @@ impl_song_properties (RBSource *asource)
 		rb_debug ("failed to create dialog, or no selection!");
 }
 
+static RhythmDBPropType
+rb_playlist_source_drag_atom_to_prop (GdkAtom smasher)
+{
+	if (smasher == gdk_atom_intern ("text/x-rhythmbox-album", TRUE))
+		return RHYTHMDB_PROP_ALBUM;
+	else if (smasher == gdk_atom_intern ("text/x-rhythmbox-artist", TRUE))
+		return RHYTHMDB_PROP_ARTIST;
+	else if (smasher == gdk_atom_intern ("text/x-rhythmbox-genre", TRUE))
+		return RHYTHMDB_PROP_GENRE;
+	else {
+		g_assert_not_reached ();
+		return 0;
+	}
+}
+
 static gboolean
 impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 {
 	GList *list;
 	RBPlaylistSource *source = RB_PLAYLIST_SOURCE (asource);
-	list = gnome_vfs_uri_list_parse (data->data);
 
-	if (list != NULL)
-		rb_playlist_source_add_list_uri (source, list);
-	else
-		return FALSE;
+        if (data->type == gdk_atom_intern ("text/uri-list", TRUE)) {
+                list = gnome_vfs_uri_list_parse ((char *) data->data);
 
-	return TRUE;
+                if (list != NULL)
+                        rb_playlist_source_add_list_uri (source, list);
+                else
+                        return FALSE;
+
+	} else {
+		GPtrArray *query;
+		GPtrArray *subquery;
+
+		subquery = rhythmdb_query_parse (source->priv->db,
+						 RHYTHMDB_QUERY_PROP_EQUALS,
+						 rb_playlist_source_drag_atom_to_prop (data->type),
+						 data->data,
+						 RHYTHMDB_QUERY_END);
+		query = rhythmdb_query_parse (source->priv->db,
+					      RHYTHMDB_QUERY_PROP_EQUALS,
+					      RHYTHMDB_PROP_TYPE,
+					      RHYTHMDB_ENTRY_TYPE_SONG,
+					      RHYTHMDB_QUERY_SUBQUERY,
+					      subquery,
+					      RHYTHMDB_QUERY_END);
+                rb_playlist_source_set_query (source, query, 0, 0);
+	}
+
+        return TRUE;
 }
 
 static gboolean
-impl_show_popup (RBSource *source)
+impl_show_popup (RBSource *asource)
 {
-	rb_bonobo_show_popup (GTK_WIDGET (source), PLAYLIST_SOURCE_POPUP_PATH);
+	RBPlaylistSource *source = RB_PLAYLIST_SOURCE (asource);
+
+	if (source->priv->automatic) {
+		_rb_source_show_popup (RB_SOURCE (asource), PLAYLIST_SOURCE_AUTOMATIC_POPUP_PATH);
+	} else {
+		_rb_source_show_popup (RB_SOURCE (asource), PLAYLIST_SOURCE_POPUP_PATH);
+	}
 	return TRUE;
 }
 
@@ -591,9 +679,7 @@ rb_playlist_source_add_location (RBPlaylistSource *source,
 {
 	RhythmDBEntry *entry;
 
-	rhythmdb_read_lock (source->priv->db);
 	if (g_hash_table_lookup (source->priv->entries, location)) {
-		rhythmdb_read_unlock (source->priv->db);
 		return;
 	}
 
@@ -602,12 +688,13 @@ rb_playlist_source_add_location (RBPlaylistSource *source,
 
 	entry = rhythmdb_entry_lookup_by_location (source->priv->db, location);
 	if (entry)
-		rhythmdb_entry_ref_unlocked (source->priv->db, entry);
-	rhythmdb_read_unlock (source->priv->db);
+		rhythmdb_entry_ref (source->priv->db, entry);
 
 	if (entry != NULL) {
 		rhythmdb_query_model_add_entry (source->priv->model, entry);
 		rhythmdb_entry_unref (source->priv->db, entry);
+
+		source->priv->dirty = TRUE;
 	}
 }
 
@@ -620,39 +707,25 @@ rb_playlist_source_remove_location (RBPlaylistSource *source,
 	g_return_if_fail (g_hash_table_lookup (source->priv->entries, location) != NULL);
 	g_hash_table_remove (source->priv->entries,
 			     location);
-	rhythmdb_read_lock (source->priv->db);
 	entry = rhythmdb_entry_lookup_by_location (source->priv->db, location);
-	rhythmdb_read_unlock (source->priv->db);
-	if (entry != NULL)
+	if (entry != NULL) {
 		rhythmdb_query_model_remove_entry (source->priv->model, entry);
+		source->priv->dirty = TRUE;
+	}
 }
 
 void
 rb_playlist_source_add_entry (RBPlaylistSource *source,
 			      RhythmDBEntry *entry)
 {
-	const char *location;
-
-	rhythmdb_read_lock (source->priv->db);
-	location = rhythmdb_entry_get_string (source->priv->db, entry,
-					      RHYTHMDB_PROP_LOCATION);
-	rhythmdb_read_unlock (source->priv->db);
-
-	rb_playlist_source_add_location (source, location);
+	rb_playlist_source_add_location (source, entry->location);
 }
 
 void
 rb_playlist_source_remove_entry (RBPlaylistSource *source,
 			         RhythmDBEntry *entry)
 {
-	const char *location;
-
-	rhythmdb_read_lock (source->priv->db);
-	location = rhythmdb_entry_get_string (source->priv->db, entry,
-					      RHYTHMDB_PROP_LOCATION);
-	rhythmdb_read_unlock (source->priv->db);
-
-	rb_playlist_source_remove_location (source, location);
+	rb_playlist_source_remove_location (source, entry->location);
 }
 
 static void
@@ -674,9 +747,8 @@ rb_playlist_source_add_list_uri (RBPlaylistSource *source,
 	for (i = uri_list; i != NULL; i = i->next) {
 		char *uri = i->data;
 		if (uri != NULL) {
-			rhythmdb_read_lock (source->priv->db);
+			rhythmdb_add_uri (source->priv->db, uri);
 			rb_playlist_source_add_location (source, uri);
-			rhythmdb_read_unlock (source->priv->db);
 		}
 
 		g_free (uri);
@@ -686,77 +758,128 @@ rb_playlist_source_add_list_uri (RBPlaylistSource *source,
 }
 
 static void
-playlist_iter_func (GtkTreeModel *model, GtkTreeIter *iter, char **uri, char **title)
+playlist_iter_func (GtkTreeModel *model, GtkTreeIter *iter, char **uri, char **title, gpointer user_data)
 {
-	RhythmDB *db;
 	RhythmDBEntry *entry;
 
-	g_object_get (G_OBJECT (model), "db", &db, NULL);
-	
 	gtk_tree_model_get (model, iter, 0, &entry, -1);
 
-	rhythmdb_read_lock (db);
-
-	*uri = g_strdup (rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_LOCATION));
-	*title = g_strdup (rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_TITLE));
-
-	rhythmdb_read_unlock (db);
+	*uri = g_strdup (entry->location);
+	*title = g_strdup (rb_refstring_get (entry->title));
 }
 
 void
 rb_playlist_source_save_playlist (RBPlaylistSource *source, const char *uri)
 {
-	RBPlaylist *playlist;
+	TotemPlParser *playlist;
 	GError *error = NULL;
 	rb_debug ("saving playlist");
 
-	playlist = rb_playlist_new ();
+	playlist = totem_pl_parser_new ();
 
-	rb_playlist_write (playlist, GTK_TREE_MODEL (source->priv->model),
-			   playlist_iter_func, uri, &error);
+	totem_pl_parser_write (playlist, GTK_TREE_MODEL (source->priv->model),
+			       playlist_iter_func, uri, TOTEM_PL_PARSER_PLS,
+			       NULL, &error);
 	if (error != NULL)
-		rb_error_dialog ("%s", error->message);
+		rb_error_dialog (NULL, _("Couldn't save playlist"),
+				 "%s", error->message);
+}
+
+#if defined(WITH_CD_BURNER_SUPPORT)
+static void
+burn_playlist_iter_func (GtkTreeModel *model, GtkTreeIter *iter, char **uri, char **artist, char **title, gulong *duration)
+{
+	RhythmDBEntry *entry;
+
+	gtk_tree_model_get (model, iter, 0, &entry, -1);
+
+	*uri = g_strdup (entry->location);
+	*artist = g_strdup (rb_refstring_get (entry->artist));
+	*title = g_strdup (rb_refstring_get (entry->title));
+	*duration = entry->duration;
+}
+#endif
+
+void
+rb_playlist_source_burn_playlist (RBPlaylistSource *source)
+{
+#if defined(WITH_CD_BURNER_SUPPORT)
+	GtkWidget *recorder;
+	char *name;
+
+	rb_debug ("burning playlist");
+
+	g_object_get (source, "name", &name, NULL);
+
+	recorder = rb_playlist_source_recorder_new (gtk_widget_get_toplevel (GTK_WIDGET (source)),
+						    name);
+	g_free (name);
+
+	rb_playlist_source_recorder_add_from_model (RB_PLAYLIST_SOURCE_RECORDER (recorder),
+						    GTK_TREE_MODEL (source->priv->model),
+						    burn_playlist_iter_func,
+						    NULL);
+
+        g_signal_connect (recorder,
+			  "response",
+			  G_CALLBACK (gtk_widget_destroy),
+			  NULL);
+
+	gtk_widget_show (recorder);
+#endif
 }
 
 RBSource *
-rb_playlist_source_new_from_xml	(RhythmDB *db,
+rb_playlist_source_new_from_xml	(RBShell *shell,
 				 xmlNodePtr node)
 {
 	RBPlaylistSource *source;
 	xmlNodePtr child;
-	char *tmp;
+	xmlChar *tmp;
 
-	source = RB_PLAYLIST_SOURCE (rb_playlist_source_new (db, FALSE));
+	source = RB_PLAYLIST_SOURCE (rb_playlist_source_new (shell, FALSE));
 
-	tmp = xmlGetProp (node, "type");
-	if (!strcmp (tmp, "automatic"))
+	tmp = xmlGetProp (node, RB_PLAYLIST_TYPE);
+	if (!xmlStrcmp (tmp, RB_PLAYLIST_AUTOMATIC))
 		source->priv->automatic = TRUE;
 	g_free (tmp);
-	tmp = xmlGetProp (node, "name");
+
+	tmp = xmlGetProp (node, RB_PLAYLIST_NAME);
 	g_object_set (G_OBJECT (source), "name", tmp, NULL);
 	g_free (tmp);
 
+	tmp = xmlGetProp (node, RB_PLAYLIST_INT_NAME);
+	if (!tmp) {
+		GTimeVal serial;
+		/* Hm.  Upgrades. */
+		g_get_current_time (&serial);
+		tmp = (xmlChar *) g_strdup_printf ("<playlist:%ld:%ld>",
+				       serial.tv_sec, serial.tv_usec);
+	}
+	g_object_set (G_OBJECT (source), "internal-name", tmp, NULL);
+	g_free (tmp);
+
 	if (source->priv->automatic) {
 		GPtrArray *query;
-		gchar *limit_str;
-		guint limit_count = 0;
-		guint limit_mb = 0;
+		xmlChar *limit_str;
+		gint limit_count = 0;
+		gint limit_mb = 0;
 
 		child = node->children;
 		while (xmlNodeIsText (child))
 			child = child->next;
 
-		query = rhythmdb_query_deserialize (db, child);
-		limit_str = xmlGetProp (node, "limit-count");
+		query = rhythmdb_query_deserialize (source->priv->db, child);
+		limit_str = xmlGetProp (node, RB_PLAYLIST_LIMIT_COUNT);
 		if (!limit_str) /* Backwards compatibility */
-			limit_str = xmlGetProp (node, "limit");
+			limit_str = xmlGetProp (node, RB_PLAYLIST_LIMIT);
 		if (limit_str) {
-			limit_count = atoi (limit_str);
+			limit_count = atoi ((char *) limit_str);
 			g_free (limit_str);
 		}
-		limit_str = xmlGetProp (node, "limit-size");
+		limit_str = xmlGetProp (node, RB_PLAYLIST_LIMIT_SIZE);
 		if (limit_str) {
-			limit_mb = atoi (limit_str);
+			limit_mb = atoi ((char *) limit_str);
 			g_free (limit_str);
 		}
 		rb_playlist_source_set_query (source, query,
@@ -764,16 +887,17 @@ rb_playlist_source_new_from_xml	(RhythmDB *db,
 					      limit_mb);
 	} else {
 		for (child = node->children; child; child = child->next) {
-			char *location;
+			xmlChar *location;
 
 			if (xmlNodeIsText (child))
 				continue;
 		
-			if (strcmp (child->name, "location"))
+			if (xmlStrcmp (child->name, RB_PLAYLIST_LOCATION))
 				continue;
 		
 			location = xmlNodeGetContent (child);
-			rb_playlist_source_add_location (source, location);
+			rb_playlist_source_add_location (source,
+							 (char *) location);
 		}
 	}
 
@@ -784,34 +908,30 @@ void
 rb_playlist_source_save_to_xml (RBPlaylistSource *source, xmlNodePtr parent_node)
 {
 	xmlNodePtr node;
-	char *name;
+	xmlChar *name;
+	xmlChar *internal_name;
 	GtkTreeIter iter;
-	
-	node = xmlNewChild (parent_node, NULL, "playlist", NULL);
-	g_object_get (G_OBJECT (source), "name", &name, NULL);
-	xmlSetProp (node, "name", name);
-	xmlSetProp (node, "type", source->priv->automatic ? "automatic" : "static");
-	
+
+	node = xmlNewChild (parent_node, NULL, RB_PLAYLIST_PLAYLIST, NULL);
+	g_object_get (G_OBJECT (source), "name", &name,
+		      "internal-name", &internal_name, NULL);
+	xmlSetProp (node, RB_PLAYLIST_NAME, name);
+	xmlSetProp (node, RB_PLAYLIST_INT_NAME, internal_name);
+	xmlSetProp (node, RB_PLAYLIST_TYPE, source->priv->automatic ? RB_PLAYLIST_AUTOMATIC : RB_PLAYLIST_STATIC);
+
 	if (!source->priv->automatic) {
 		if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (source->priv->model),
 						    &iter))
 			return;
-
 		do { 
-			xmlNodePtr child_node = xmlNewChild (node, NULL, "location", NULL);
+			xmlNodePtr child_node = xmlNewChild (node, NULL, RB_PLAYLIST_LOCATION, NULL);
 			RhythmDBEntry *entry;
-			const char *location;
-			char *encoded;
+			xmlChar *encoded;
 
 			gtk_tree_model_get (GTK_TREE_MODEL (source->priv->model), &iter, 0, &entry, -1);
 
-			rhythmdb_read_lock (source->priv->db);
-			location = rhythmdb_entry_get_string (source->priv->db, entry,
-							      RHYTHMDB_PROP_LOCATION);
-			rhythmdb_read_unlock (source->priv->db);
-			
-			encoded = xmlEncodeEntitiesReentrant (NULL, location);			
-			
+			encoded = xmlEncodeEntitiesReentrant (NULL, BAD_CAST entry->location);
+
 			xmlNodeSetContent (child_node, encoded);
 			g_free (encoded);
 		} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (source->priv->model),
@@ -827,11 +947,13 @@ rb_playlist_source_save_to_xml (RBPlaylistSource *source, xmlNodePtr parent_node
 			      "max-size", &max_size_mb,
 			      "query", &query, NULL);
 		limit_str = g_strdup_printf ("%d", max_count);
-		xmlSetProp (node, "limit-count", limit_str);
+		xmlSetProp (node, RB_PLAYLIST_LIMIT_COUNT, BAD_CAST limit_str);
 		g_free (limit_str);
 		limit_str = g_strdup_printf ("%d", max_size_mb);
-		xmlSetProp (node, "limit-size", limit_str);
+		xmlSetProp (node, RB_PLAYLIST_LIMIT_SIZE, BAD_CAST limit_str);
 		g_free (limit_str);
 		rhythmdb_query_serialize (source->priv->db, query, node);
 	}
+
+	source->priv->dirty = FALSE;
 }
