@@ -1,6 +1,8 @@
 /*
+ *  arch-tag: Header for the abstract base class of all sources
+ *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
- *  Copyright (C) 2003 Colin Walters <cwalters@gnome.org>
+ *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,16 +18,15 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-source.h,v 1.9 2003/08/17 07:34:19 cwalters Exp $
  */
 
 #ifndef __RB_SOURCE_H
 #define __RB_SOURCE_H
 
 #include <gtk/gtkhbox.h>
-#include <bonobo/bonobo-ui-component.h>
+#include <gtk/gtkactiongroup.h>
 
-#include "rb-node-view.h"
+#include "rb-entry-view.h"
 
 G_BEGIN_DECLS
 
@@ -58,20 +59,25 @@ typedef struct
 	/* signals */
 	void (*status_changed)	(RBSource *source);
 	void (*filter_changed)	(RBSource *source);
+	void (*deleted)		(RBSource *source);
 
 	/* methods */
-	const char *	(*impl_get_status)	(RBSource *source);
+	char *	        (*impl_get_status)	(RBSource *source);
 
 	const char *	(*impl_get_browser_key)	(RBSource *source);
 
-	RBNodeView *	(*impl_get_node_view)	(RBSource *source);
+	const char *	(*impl_get_search_key)	(RBSource *source);
+
+	RBEntryView *	(*impl_get_entry_view)	(RBSource *source);
 	GList *		(*impl_get_extra_views)	(RBSource *source);
 
 	GdkPixbuf *	(*impl_get_pixbuf)	(RBSource *source);
+	gboolean	(*impl_can_rename)	(RBSource *source);
 
 	gboolean	(*impl_can_search)	(RBSource *source);
 
 	void		(*impl_search)		(RBSource *source, const char *text);
+	void		(*impl_reset_filters)	(RBSource *source);
 	GtkWidget *	(*impl_get_config_widget)(RBSource *source);
 
 	gboolean	(*impl_can_cut)		(RBSource *source);
@@ -80,7 +86,7 @@ typedef struct
 	
 	GList *		(*impl_cut)		(RBSource *source);
 	GList *		(*impl_copy)		(RBSource *source);
-	void		(*impl_paste)		(RBSource *source, GList *nodes);
+	void		(*impl_paste)		(RBSource *source, GList *entries);
 	void		(*impl_delete)		(RBSource *source);
 
 	void		(*impl_song_properties)	(RBSource *source);
@@ -92,11 +98,10 @@ typedef struct
 	const char *	(*impl_get_artist)	(RBSource *source);
 	const char *	(*impl_get_album)	(RBSource *source);
 	gboolean	(*impl_have_url)	(RBSource *source);
-	void		(*impl_buffering_done)	(RBSource *source);
-
 	gboolean	(*impl_receive_drag)	(RBSource *source, GtkSelectionData *data);
 	gboolean	(*impl_show_popup)	(RBSource *source);
 				   
+	void		(*impl_delete_thyself)	(RBSource *source);
 } RBSourceClass;
 
 typedef gboolean (*RBSourceFeatureFunc) (RBSource *source);
@@ -107,22 +112,30 @@ void		rb_source_notify_filter_changed	(RBSource *source);
 
 void		rb_source_notify_status_changed (RBSource *status);
 
+void		rb_source_update_play_statistics(RBSource *source, RhythmDB *db,
+						 RhythmDBEntry *entry);
+
 /* general interface */
-const char *	rb_source_get_status		(RBSource *source);
+char *	        rb_source_get_status		(RBSource *source);
 
 const char *	rb_source_get_browser_key	(RBSource *source);
 
-RBNodeView *	rb_source_get_node_view		(RBSource *source);
+const char *	rb_source_get_search_key	(RBSource *source);
+
+RBEntryView *	rb_source_get_entry_view	(RBSource *source);
 
 GList *		rb_source_get_extra_views	(RBSource *source);
 
 GdkPixbuf *	rb_source_get_pixbuf		(RBSource *source);
+gboolean	rb_source_can_rename		(RBSource *source);
 
 gboolean	rb_source_can_search		(RBSource *source);
 
 void		rb_source_search		(RBSource *source,
 						 const char *text);
 
+void		rb_source_reset_filters		(RBSource *source);
+
 GtkWidget *	rb_source_get_config_widget	(RBSource *source);
 
 gboolean	rb_source_can_cut		(RBSource *source);
@@ -131,7 +144,7 @@ gboolean	rb_source_can_copy		(RBSource *source);
 
 GList *		rb_source_cut			(RBSource *source);
 GList *		rb_source_copy			(RBSource *source);
-void		rb_source_paste			(RBSource *source, GList *nodes);
+void		rb_source_paste			(RBSource *source, GList *entries);
 void		rb_source_delete		(RBSource *source);
 
 void		rb_source_song_properties	(RBSource *source);
@@ -143,12 +156,17 @@ gboolean	rb_source_have_artist_album	(RBSource *source);
 const char *	rb_source_get_artist		(RBSource *source);
 const char *	rb_source_get_album		(RBSource *source);
 gboolean	rb_source_have_url		(RBSource *source);
-void		rb_source_buffering_done	(RBSource *source);
 
 gboolean	rb_source_receive_drag		(RBSource *source, GtkSelectionData *data);
 
 gboolean	rb_source_show_popup		(RBSource *source);
 
+void		rb_source_delete_thyself	(RBSource *source);
+
+/* Protected method, should only be used by objects inheriting from RBSource */
+void            _rb_source_show_popup           (RBSource *source, 
+						 const char *ui_path);
+
 G_END_DECLS
 
 #endif /* __RB_SOURCE_H */
