@@ -1,7 +1,7 @@
 /*
  * arch-tag: Implementation of main "Sources" display widget
  *
- * Copyright (C) 2003 Colin Walters <walters@verbum.org>
+ * Copyright (C) 2003,2004 Colin Walters <walters@verbum.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -31,18 +31,32 @@
 #include "rb-sourcelist.h"
 #include "rb-sourcelist-model.h"
 #include "rb-debug.h"
+#include "rb-stock-icons.h"
 #include "rb-marshal.h"
-#include "rb-tree-view-column.h"
 #include "rb-cell-renderer-pixbuf.h"
 #include "rb-tree-dnd.h"
+#include "rb-util.h"
 
 struct RBSourceListPriv
 {
 	GtkWidget *treeview;
 	GtkCellRenderer *title_renderer;
 
-	GtkTreeModel *model;
+	GtkTreeModel *real_model;
+	GtkTreeModel *filter_model;
 	GtkTreeSelection *selection;
+
+	RBSource *playing_source;
+	GdkPixbuf *playing_pixbuf;
+	GdkPixbuf *paused_pixbuf;
+
+	RBShell *shell;
+};
+
+enum
+{
+	PROP_0,
+	PROP_SHELL
 };
 
 enum
@@ -57,6 +71,14 @@ enum
 static void rb_sourcelist_class_init (RBSourceListClass *klass);
 static void rb_sourcelist_init (RBSourceList *sourcelist);
 static void rb_sourcelist_finalize (GObject *object);
+static void rb_sourcelist_set_property (GObject *object,
+					guint prop_id,
+					const GValue *value,
+					GParamSpec *pspec);
+static void rb_sourcelist_get_property (GObject *object,
+					guint prop_id,
+					GValue *value,
+					GParamSpec *pspec);
 static void rb_sourcelist_selection_changed_cb (GtkTreeSelection *selection,
 						RBSourceList *sourcelist);
 static void drop_received_cb (RBSourceListModel *model, RBSource *target, GtkTreeViewDropPosition pos,
@@ -66,9 +88,15 @@ static void row_activated_cb (GtkTreeView *treeview, GtkTreePath *path,
 static gboolean button_press_cb (GtkTreeView *treeview,
 				 GdkEventButton *event,
 				 RBSourceList *sourcelist);
+static gboolean popup_menu_cb (GtkTreeView *treeview, RBSourceList *sourcelist);
 static void name_notify_cb (GObject *obj, GParamSpec *pspec, gpointer data);
+static void visibility_notify_cb (GObject *obj, GParamSpec *pspec, 
+				  gpointer data);
 static void source_name_edited_cb (GtkCellRendererText *renderer, const char *pathstr,
 				   const char *text, RBSourceList *sourcelist);
+static void rb_sourcelist_playing_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
+						  GtkTreeModel *tree_model, GtkTreeIter *iter,
+						  RBSourceList *sourcelist);
 static void rb_sourcelist_title_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
 						GtkTreeModel *tree_model, GtkTreeIter *iter,
 						RBSourceList *sourcelist);
@@ -115,6 +143,16 @@ rb_sourcelist_class_init (RBSourceListClass *class)
 	object_class = (GtkObjectClass *) class;
 
 	o_class->finalize = rb_sourcelist_finalize;
+	o_class->set_property = rb_sourcelist_set_property;
+	o_class->get_property = rb_sourcelist_get_property;
+
+	g_object_class_install_property (o_class,
+					 PROP_SHELL,
+					 g_param_spec_object ("shell",
+							      "RBShell",
+							      "RBShell object",
+							      RB_TYPE_SHELL,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	rb_sourcelist_signals[SELECTED] =
 		g_signal_new ("selected",
@@ -166,17 +204,19 @@ rb_sourcelist_init (RBSourceList *sourcelist)
 {
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *gcolumn;
+	GtkWidget *dummy;
 
 	sourcelist->priv = g_new0 (RBSourceListPriv, 1);
 
-	sourcelist->priv->model = rb_sourcelist_model_new ();
+	sourcelist->priv->filter_model = rb_sourcelist_model_new ();
+	sourcelist->priv->real_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (sourcelist->priv->filter_model));
 
-	g_signal_connect_object (G_OBJECT (sourcelist->priv->model),
+	g_signal_connect_object (G_OBJECT (sourcelist->priv->filter_model),
 				 "drop_received",
 				 G_CALLBACK (drop_received_cb),
 				 sourcelist, 0);
 
-	sourcelist->priv->treeview = gtk_tree_view_new_with_model (sourcelist->priv->model);
+	sourcelist->priv->treeview = gtk_tree_view_new_with_model (sourcelist->priv->filter_model);
 	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (sourcelist->priv->treeview), FALSE);
 
 	g_signal_connect_object (G_OBJECT (sourcelist->priv->treeview),
@@ -189,13 +229,30 @@ rb_sourcelist_init (RBSourceList *sourcelist)
 				 G_CALLBACK (button_press_cb),
 				 sourcelist, 0);
 
-	/* Set up the pixbuf column */
-	renderer = gtk_cell_renderer_pixbuf_new ();
+	g_signal_connect_object (G_OBJECT (sourcelist->priv->treeview),
+				 "popup_menu",
+				 G_CALLBACK (popup_menu_cb),
+				 sourcelist, 0);
+
+	dummy = gtk_tree_view_new ();
+	sourcelist->priv->playing_pixbuf = gtk_widget_render_icon (dummy,
+								   RB_STOCK_PLAYING,
+								   GTK_ICON_SIZE_MENU,
+								   NULL);
+
+	sourcelist->priv->paused_pixbuf = gtk_widget_render_icon (dummy,
+								  RB_STOCK_PAUSED,
+								  GTK_ICON_SIZE_MENU,
+								  NULL);
+	gtk_widget_destroy (dummy);
+
 	gcolumn = gtk_tree_view_column_new ();
 	gtk_tree_view_column_set_title (gcolumn, _("_Source"));
 	gtk_tree_view_column_set_clickable (gcolumn, FALSE);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (sourcelist->priv->treeview), gcolumn);
 
+	/* Set up the pixbuf column */
+	renderer = gtk_cell_renderer_pixbuf_new ();
 	gtk_tree_view_column_pack_start (gcolumn, renderer, FALSE);
 	gtk_tree_view_column_set_attributes (gcolumn, renderer,
 				             "pixbuf", RB_SOURCELIST_MODEL_COLUMN_PIXBUF,
@@ -210,6 +267,14 @@ rb_sourcelist_init (RBSourceList *sourcelist)
 						 sourcelist, NULL);
 	g_signal_connect_object (renderer, "edited", G_CALLBACK (source_name_edited_cb), sourcelist, 0);
 
+	/* Set up the playing column */
+	renderer = rb_cell_renderer_pixbuf_new ();
+	gtk_tree_view_column_pack_start (gcolumn, renderer, FALSE);
+	gtk_tree_view_column_set_cell_data_func (gcolumn, renderer,
+						 (GtkTreeCellDataFunc)
+						 rb_sourcelist_playing_cell_data_func,
+						 sourcelist, NULL);
+
 	sourcelist->priv->selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (sourcelist->priv->treeview));
 	g_signal_connect_object (G_OBJECT (sourcelist->priv->selection),
 			         "changed",
@@ -223,14 +288,53 @@ rb_sourcelist_finalize (GObject *object)
 {
 	RBSourceList *sourcelist = RB_SOURCELIST (object);
 
+	g_object_unref (G_OBJECT (sourcelist->priv->playing_pixbuf));
+	g_object_unref (G_OBJECT (sourcelist->priv->paused_pixbuf));
+
 	g_free (sourcelist->priv);
 
 	if (G_OBJECT_CLASS (parent_class)->finalize)
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
+static void
+rb_sourcelist_set_property (GObject *object,
+			    guint prop_id,
+			    const GValue *value,
+			    GParamSpec *pspec)
+{
+	RBSourceList *sourcelist = RB_SOURCELIST (object);
+	switch (prop_id)
+	{
+	case PROP_SHELL:
+		sourcelist->priv->shell = g_value_get_object (value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void 
+rb_sourcelist_get_property (GObject *object,
+			    guint prop_id,
+			    GValue *value,
+			    GParamSpec *pspec)
+{
+	RBSourceList *sourcelist = RB_SOURCELIST (object);
+	switch (prop_id)
+	{
+	case PROP_SHELL:
+		g_value_set_object (value, sourcelist->priv->shell);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
 
 GtkWidget *
-rb_sourcelist_new (void)
+rb_sourcelist_new (RBShell *shell)
 {
 	RBSourceList *sourcelist;
 
@@ -240,6 +344,7 @@ rb_sourcelist_new (void)
 					          "hscrollbar_policy", GTK_POLICY_AUTOMATIC,
 					          "vscrollbar_policy", GTK_POLICY_AUTOMATIC,
 					          "shadow_type", GTK_SHADOW_IN,
+						  "shell", shell,
 					          NULL));
 
 	gtk_container_add (GTK_CONTAINER (sourcelist),
@@ -261,43 +366,95 @@ rb_sourcelist_append (RBSourceList *sourcelist,
 
 	g_object_get (G_OBJECT (source), "name", &name, NULL);
 
-	gtk_list_store_append (GTK_LIST_STORE (sourcelist->priv->model), &iter);
+	gtk_list_store_append (GTK_LIST_STORE (sourcelist->priv->real_model), &iter);
 
-	gtk_list_store_set (GTK_LIST_STORE (sourcelist->priv->model), &iter,
+	gtk_list_store_set (GTK_LIST_STORE (sourcelist->priv->real_model), &iter,
 			    RB_SOURCELIST_MODEL_COLUMN_PIXBUF, rb_source_get_pixbuf (source),
 			    RB_SOURCELIST_MODEL_COLUMN_NAME, name,
 			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, source,
 			    RB_SOURCELIST_MODEL_COLUMN_ATTRIBUTES, attrs,
 			    -1);
 
-	g_signal_connect_object (G_OBJECT (source), "notify", G_CALLBACK (name_notify_cb), sourcelist, 0);
+	g_signal_connect_object (G_OBJECT (source), "notify::name", G_CALLBACK (name_notify_cb), sourcelist, 0);
+	g_signal_connect_object (G_OBJECT (source), "notify::visibility", G_CALLBACK (visibility_notify_cb), sourcelist, 0);
+
+}
+
+static gboolean
+rb_sourcelist_source_to_iter (RBSourceList *sourcelist, RBSource *source,
+			      GtkTreeIter *iter)
+{
+	if (gtk_tree_model_get_iter_first (sourcelist->priv->real_model, iter) == FALSE) {
+		return FALSE;
+	}
+
+	do {
+		RBSource *target = NULL;
+		gtk_tree_model_get (sourcelist->priv->real_model, iter,
+				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
+		if (source == target)
+			return TRUE;
+	} while (gtk_tree_model_iter_next (sourcelist->priv->real_model, iter));
+	return FALSE;
+}
+
+static gboolean
+rb_sourcelist_visible_source_to_iter (RBSourceList *sourcelist, RBSource *source,
+				      GtkTreeIter *iter)
+{
+	if (gtk_tree_model_get_iter_first (sourcelist->priv->filter_model, iter) == FALSE) {
+		return FALSE;
+	}
+	do {
+		RBSource *target = NULL;
+		gtk_tree_model_get (sourcelist->priv->filter_model, iter,
+				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
+		if (source == target)
+			return TRUE;
+	} while (gtk_tree_model_iter_next (sourcelist->priv->filter_model, iter));
+	return FALSE;
 }
 
 void
 rb_sourcelist_edit_source_name (RBSourceList *sourcelist, RBSource *source)
 {
 	GtkTreeIter iter;
+	GtkTreePath *path;
+	GtkTreeViewColumn *col;
+	
+	g_assert (rb_sourcelist_visible_source_to_iter (sourcelist, source, &iter));
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (sourcelist->priv->filter_model),
+					&iter);
+	col = gtk_tree_view_get_column (GTK_TREE_VIEW (sourcelist->priv->treeview), 0);
+	
+	/* Make cell editable just for the moment.
+	   We'll turn it off once editing is done. */
+	g_object_set (G_OBJECT (sourcelist->priv->title_renderer), "editable", TRUE, NULL);
+	
+	gtk_tree_view_set_cursor_on_cell (GTK_TREE_VIEW (sourcelist->priv->treeview),
+					  path, col, sourcelist->priv->title_renderer,
+					  TRUE);
+	
+	gtk_tree_path_free (path);
+}
 
-	gtk_tree_model_get_iter_first (sourcelist->priv->model, &iter);
-	do {
-		gpointer target = NULL;
-		gtk_tree_model_get (sourcelist->priv->model, &iter,
-				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
-		if (source == target) {
-			GtkTreePath *path = gtk_tree_model_get_path (GTK_TREE_MODEL (sourcelist->priv->model),
-								     &iter);
-			GtkTreeViewColumn *col;
-
-			col = gtk_tree_view_get_column (GTK_TREE_VIEW (sourcelist->priv->treeview), 0);
-			gtk_tree_view_set_cursor_on_cell (GTK_TREE_VIEW (sourcelist->priv->treeview),
-							  path, col, sourcelist->priv->title_renderer,
-							  TRUE);
-
-			gtk_tree_path_free (path);
-			return;
-		}
-	} while (gtk_tree_model_iter_next (sourcelist->priv->model, &iter));
-	g_assert_not_reached ();
+void
+rb_sourcelist_set_playing_source (RBSourceList *sourcelist, RBSource *source)
+{
+	GtkTreeIter iter;
+	if (sourcelist->priv->playing_source) {
+		g_assert (rb_sourcelist_source_to_iter (sourcelist,
+							sourcelist->priv->playing_source,
+							&iter));
+		gtk_list_store_set (GTK_LIST_STORE (sourcelist->priv->real_model), &iter,
+				    RB_SOURCELIST_MODEL_COLUMN_PLAYING, FALSE, -1);
+	}
+	sourcelist->priv->playing_source = source;
+	if (source) {
+		g_assert (rb_sourcelist_source_to_iter (sourcelist, source, &iter));
+		gtk_list_store_set (GTK_LIST_STORE (sourcelist->priv->real_model), &iter,
+				    RB_SOURCELIST_MODEL_COLUMN_PLAYING, TRUE, -1);
+	}
 }
 
 void
@@ -305,18 +462,13 @@ rb_sourcelist_remove (RBSourceList *sourcelist, RBSource *source)
 {
 	GtkTreeIter iter;
 
-	gtk_tree_model_get_iter_first (sourcelist->priv->model, &iter);
-	do {
-		gpointer target = NULL;
-		gtk_tree_model_get (sourcelist->priv->model, &iter,
-				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
-		if (source == target) {
-			gtk_list_store_remove (GTK_LIST_STORE (sourcelist->priv->model), &iter);
-			g_signal_handlers_disconnect_by_func (G_OBJECT (source), G_CALLBACK (name_notify_cb), sourcelist);
-			return;
-		}
-	} while (gtk_tree_model_iter_next (sourcelist->priv->model, &iter));
-	g_assert_not_reached ();
+	g_assert (rb_sourcelist_source_to_iter (sourcelist, source, &iter));
+	gtk_list_store_remove (GTK_LIST_STORE (sourcelist->priv->real_model), &iter);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (source),
+					      G_CALLBACK (name_notify_cb), sourcelist);
+        g_signal_handlers_disconnect_by_func (G_OBJECT (source),
+					      G_CALLBACK (visibility_notify_cb),
+                                             sourcelist);
 }
 
 void
@@ -324,17 +476,30 @@ rb_sourcelist_select (RBSourceList *sourcelist, RBSource *source)
 {
 	GtkTreeIter iter;
 
-	gtk_tree_model_get_iter_first (sourcelist->priv->model, &iter);
-	do {
-		gpointer target = NULL;
-		gtk_tree_model_get (sourcelist->priv->model, &iter,
-				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
-		if (source == target) {
-			gtk_tree_selection_select_iter (sourcelist->priv->selection, &iter);
-			return;
-		}
-	} while (gtk_tree_model_iter_next (sourcelist->priv->model, &iter));
-	g_assert_not_reached ();
+	g_assert (rb_sourcelist_visible_source_to_iter (sourcelist, source, &iter));
+	gtk_tree_selection_select_iter (sourcelist->priv->selection, &iter);
+}
+
+static void
+rb_sourcelist_playing_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
+				      GtkTreeModel *tree_model, GtkTreeIter *iter,
+				      RBSourceList *sourcelist)
+{
+	GdkPixbuf *pixbuf;
+	gboolean playing;
+	gtk_tree_model_get (sourcelist->priv->filter_model, iter,
+			    RB_SOURCELIST_MODEL_COLUMN_PLAYING, &playing, -1);
+	
+	if (playing)
+		pixbuf = sourcelist->priv->playing_pixbuf;
+#if 0
+	else if (source == sourcelist->priv->playing_source)
+		pixbuf = sourcelist->priv->paused_pixbuf;
+#endif
+	else
+		pixbuf = NULL;
+
+	g_object_set (G_OBJECT (renderer), "pixbuf", pixbuf, NULL);
 }
 
 static void
@@ -349,7 +514,7 @@ rb_sourcelist_selection_changed_cb (GtkTreeSelection *selection,
 	if (!gtk_tree_selection_get_selected (sourcelist->priv->selection,
 					      &cindy, &iter))
 		return;
-
+	
 	gtk_tree_model_get (cindy, &iter,
 			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
 	g_return_if_fail (RB_IS_SOURCE (target));
@@ -402,22 +567,18 @@ row_activated_cb (GtkTreeView *treeview,
 }
 
 static gboolean
-button_press_cb (GtkTreeView *treeview,
-		 GdkEventButton *event,
+emit_show_popup (GtkTreeView *treeview,
 		 RBSourceList *sourcelist)
 {
 	GtkTreeIter iter;
 	RBSource *target;
 	gboolean ret;
 
-	if (event->button != 3)
-		return FALSE;
-
 	if (!gtk_tree_selection_get_selected (gtk_tree_view_get_selection (treeview),
 					      NULL, &iter))
 		return FALSE;
 
-	gtk_tree_model_get (sourcelist->priv->model, &iter,
+	gtk_tree_model_get (sourcelist->priv->filter_model, &iter,
 			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
 	g_return_val_if_fail (RB_IS_SOURCE (target), FALSE);
 
@@ -426,31 +587,62 @@ button_press_cb (GtkTreeView *treeview,
 	return ret;
 }
 
+static gboolean
+button_press_cb (GtkTreeView *treeview,
+		 GdkEventButton *event,
+		 RBSourceList *sourcelist)
+{
+	GtkTreeIter iter;
+	GtkTreePath *path;
+	
+	if (event->button != 3)
+		return FALSE;
+
+	if (gtk_tree_view_get_path_at_pos (treeview, event->x, event->y,
+					   &path, NULL, NULL, NULL) == FALSE) {
+		/* pointer is over empty space */
+		GtkUIManager *uimanager;
+		g_object_get (G_OBJECT (sourcelist->priv->shell), "ui-manager", &uimanager, NULL);
+		rb_gtk_action_popup_menu (uimanager, "/SourceListPopup");
+		g_object_unref (G_OBJECT (uimanager));
+		return TRUE;
+	}
+
+	gtk_tree_model_get_iter (GTK_TREE_MODEL (sourcelist->priv->filter_model), &iter, path);
+	gtk_tree_selection_select_iter (gtk_tree_view_get_selection (treeview), &iter);
+
+	return emit_show_popup (treeview, sourcelist);
+}
+
+static gboolean
+popup_menu_cb (GtkTreeView *treeview,
+	       RBSourceList *sourcelist)
+{
+	return emit_show_popup (treeview, sourcelist);
+}
+
 static void
 name_notify_cb (GObject *obj, GParamSpec *pspec, gpointer data)
 {
 	RBSourceList *sourcelist = RB_SOURCELIST (data);
 	RBSource *source = RB_SOURCE (obj);
 	GtkTreeIter iter;
-
-	if (strcmp (g_param_spec_get_name (pspec), "name"))
-		return;
-
-	gtk_tree_model_get_iter_first (sourcelist->priv->model, &iter);
-	do {
-		gpointer target = NULL;
-		gtk_tree_model_get (sourcelist->priv->model, &iter,
-				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &target, -1);
-		if (source == target) {
-			const char *name;
-
-			g_object_get (obj, "name", &name, NULL);
-			gtk_list_store_set (GTK_LIST_STORE (sourcelist->priv->model), &iter,
-					    RB_SOURCELIST_MODEL_COLUMN_NAME, name, -1);
-			return;
-		}
-	} while (gtk_tree_model_iter_next (sourcelist->priv->model, &iter));
-	g_assert_not_reached ();
+	gchar *name;
+	
+	if (rb_sourcelist_source_to_iter (sourcelist, source, &iter)) {
+		g_object_get (obj, "name", &name, NULL);
+		gtk_list_store_set (GTK_LIST_STORE (sourcelist->priv->real_model),
+				    &iter,
+				    RB_SOURCELIST_MODEL_COLUMN_NAME, name, -1);
+	}
+}
+	
+static void
+visibility_notify_cb (GObject *obj, GParamSpec *pspec, gpointer data)
+{
+	RBSourceList *sourcelist = RB_SOURCELIST (data);
+	
+	gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (sourcelist->priv->filter_model));
 }
 
 static void
@@ -461,16 +653,11 @@ rb_sourcelist_title_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *
 	RBSource *source;
 	char *str;
 
-	gtk_tree_model_get (GTK_TREE_MODEL (sourcelist->priv->model), iter,
+	gtk_tree_model_get (GTK_TREE_MODEL (sourcelist->priv->filter_model), iter,
 			    RB_SOURCELIST_MODEL_COLUMN_NAME, &str,
 			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source, -1);
 
-	if (gtk_tree_selection_iter_is_selected (sourcelist->priv->selection, iter) != FALSE)
-		g_object_set (G_OBJECT (renderer), "text", str,
-			      "editable", rb_source_can_rename (source),
-			      NULL);
-	else
-		g_object_set (G_OBJECT (renderer), "text", str, NULL);
+	g_object_set (G_OBJECT (renderer), "text", str, NULL);
 
 	g_free (str);
 }
@@ -488,11 +675,14 @@ source_name_edited_cb (GtkCellRendererText *renderer, const char *pathstr,
 	
 	path = gtk_tree_path_new_from_string (pathstr);	
 	
-	gtk_tree_model_get_iter (GTK_TREE_MODEL (sourcelist->priv->model), &iter, path);
-	gtk_tree_model_get (sourcelist->priv->model,
+	gtk_tree_model_get_iter (GTK_TREE_MODEL (sourcelist->priv->filter_model),
+				 &iter, path);
+	gtk_tree_model_get (sourcelist->priv->filter_model,
 			    &iter, RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source, -1);
 	g_object_set (G_OBJECT (source), "name", text, NULL);
 
 	gtk_tree_path_free (path);
+
+	g_object_set (G_OBJECT (renderer), "editable", FALSE, NULL);
 }
 
