@@ -26,7 +26,7 @@
 #include <gtk/gtkscrolledwindow.h>
 
 #include "rb-source.h"
-#include "rb-sourcelist-model.h"
+#include "rb-shell.h"
 
 G_BEGIN_DECLS
 
@@ -62,12 +62,13 @@ typedef struct RBSourceListClass
 
 GType		rb_sourcelist_get_type		(void);
 
-GtkWidget *	rb_sourcelist_new		(void);
+GtkWidget *	rb_sourcelist_new		(RBShell *shell);
 
 void		rb_sourcelist_append		(RBSourceList *sourcelist,
 						 RBSource *source);
 
-GtkTreeModel *	rb_sourcelist_get_model		(RBSourceList *sourcelist);
+void		rb_sourcelist_set_playing_source(RBSourceList *sourcelist,
+						 RBSource *source);
 
 void		rb_sourcelist_edit_source_name	(RBSourceList *sourcelist,
 						 RBSource *source);
