@@ -143,7 +143,11 @@ void		rb_entry_view_enable_drag_source	(RBEntryView *view,
 							 int n_targets);
 RhythmDBEntry *	rb_entry_view_get_random_entry		(RBEntryView *view);
 
+void		rb_entry_view_get_sorting_order		(RBEntryView *view, const char **column_name, gint *sort_order);
+void		rb_entry_view_set_sorting_order		(RBEntryView *view, const char *column_name, gint sort_order);
+/* deal with the sorting order as a composite string */
 const char *	rb_entry_view_get_sorting_type		(RBEntryView *view);
+ void		rb_entry_view_set_sorting_type		(RBEntryView *view, const char *sorttype);
 
 /** Should be called by sources before resorting the query model. */
 void		rb_entry_view_set_resorting		(RBEntryView *view);
